<?php
/**
 * The HomePage utility
 *
 * @package Books
 */

defined( 'WPINC' ) || exit;

/**
 * Class for featured layouts.
 */
class Books_Homepage extends Books_Base {

	/**
	 * The Construct
	 */
	public function __construct() {
		$this->add_action( 'books_homepage_sections', 'get_homepage_sections' );
	}

	/**
	 * HomePage Sections
	 */
	public function get_homepage_sections() {

		$options = books_get_settings( 'homepage_layout' );

		if ( ! is_array( $options ) || ! array_key_exists( 'enabled', $options ) ) {
			return;
		}

		foreach ( $options['enabled'] as $template => $section ) {
			get_template_part( 'template-parts/homepage/home', $template );
		}

	}

	/**
	 * Get section title
	 *
	 * @param array $title title with tag.
	 */
	public function get_section_title( $title = [] ) {

		if ( ! $title ) {
			return;
		}

		$new_title = sprintf(
			'<%2$s class="section-title">%1$s</%2$s>',
			...array_values( $title )
		);

		return $new_title;
	}

	/**
	 * Get post title
	 */
	public function get_post_title() {
		?>
		<header>

			<h3 class="title front-view-title">
				<a href="<?php echo esc_url( get_the_permalink() ); ?>" title="<?php the_title_attribute(); ?>"><?php echo the_title(); ?></a>
			</h3>

		</header>
		<?php
	}

	/**
	 * Get post thumbnail according to layout
	 *
	 * @param String $size for thumbnail size.
	 */
	public function get_thumbnail( $size = 'books-featured' ) {
		?>
		<a href="<?php echo esc_url( get_the_permalink() ); ?>" title="<?php the_title_attribute(); ?>" class="post-image <?php echo esc_html( $size ); ?>">
			<div class="featured-thumbnail">
				<?php
				$attr = [
					'title'   => esc_attr( get_the_title() ),
					'alt'     => esc_attr( get_the_title() ),
					'loading' => 'lazy',
				];
				if ( has_post_thumbnail() ) {
					the_post_thumbnail( $size, $attr );
				} else {
					echo '<img src="' . esc_html( get_template_directory_uri() ) . '/images/nothumb-' . esc_html( $size ) . '.png" alt="' . esc_html__( 'No Preview', 'books' ) . '"  class="wp-post-image" />';
				}
				?>
			</div>
		</a>
		<?php
	}

	/**
	 * Get Meta info
	 *
	 * @param Array   $meta for post meta info.
	 * @param Boolean $blog for blog section.
	 */
	public function get_meta_info( $meta, $blog = false ) {

		if ( ! $meta ) {
			return;
		}

		$start = '<div class="post-meta">';
		$end   = '</div>';

		if ( ! $blog ) {

			$terms = get_the_terms( get_the_ID(), 'pa_author-name' );

			if ( ! isset( $meta['date'] ) && ( ! $terms || ! is_array( $terms ) ) ) {
				$start = '';
				$end   = '';
			}
		}

		echo wp_kses_post( $start );

		if ( isset( $meta['date'] ) ) {
			echo '<span class="thetime date updated">' . esc_html__( 'Posted ', 'books' ) . get_the_time( get_option( 'date_format' ) ) . '</span>'; // @codingStandardsIgnoreLine
		}

		if ( isset( $meta['author'] ) ) {
			if ( $blog ) {
				echo '<span class="theauthor">' . esc_html__( 'By ', 'books' ) . get_the_author_posts_link() . '</span>'; // @codingStandardsIgnoreLine
			} else {
				if ( $terms && is_array( $terms ) ) {
					foreach ( $terms as $term ) {
						echo '<span class="theauthor">' . esc_html__( 'By ', 'books' ) . $term->name . '</span>'; // @codingStandardsIgnoreLine
					}
				}
			}
		}

		echo wp_kses_post( $end );

	}

	/**
	 * Get Content
	 *
	 * @param Number $excerpt_length length of excerpt in words.
	 */
	public function get_content( $excerpt_length = '30' ) {
		?>
			<div class="front-view-content">
				<?php echo books_truncate( get_the_content(), $excerpt_length, 'words' ); // phpcs:ignore ?>
			</div>

		<?php
	}

	/**
	 * Return "More Details" HTML with a link.
	 */
	public function get_custom_readmore() {

		$icon = '<i class="fa fa-angle-right"></i>';

		$output  = '<a href="' . get_the_permalink() . '">';
		$output .= '<div class="overlay">';
		$output .= '<div class="readMore">';
		$output .= __( 'More details', 'books' ) . $icon;
		$output .= '</div>';
		$output .= '</div>';

		echo $output; // phpcs:ignore
	}

	/**
	 * Get default query args
	 *
	 * @param Array $new_args for additional args.
	 */
	public function get_default_args( $new_args = [] ) {

		$args = [
			'post_type'      => 'product',
			'post_status'    => 'publish',
			'posts_per_page' => -1,
			'order'          => 'DESC',
		];

		return array_merge( $args, $new_args );

	}

}

// Init.
new Books_Homepage();
