<?php
/**
 * Dynamic CSS for the frontend.
 *
 * @package Books
 */

defined( 'WPINC' ) || exit;

/**
 * Helper function.
 * Merge and combine the CSS elements.
 *
 * @param  string|array $elements An array of our elements.
 *                                If we use a string then it is directly returned.
 * @return string
 */
function books_implode( $elements = [] ) {

	if ( ! is_array( $elements ) ) {
		return $elements;
	}

	// Make sure our values are unique.
	$elements = array_unique( $elements );

	// Sort elements alphabetically.
	// This way all duplicate items will be merged in the final CSS array.
	sort( $elements );

	// Implode items and return the value.
	return implode( ',', $elements );

}

/**
 * Maps elements from dynamic css to the selector.
 *
 * @param  array  $elements The elements.
 * @param  string $selector The selector.
 * @return array
 */
function books_map_selector( $elements, $selector ) {
	$array = [];

	foreach ( $elements as $element ) {
		$array[] = $element . $selector;
	}

	return $array;
}

/**
 * Map CSS selectors from values.
 *
 * @param array $css    Array of dynamic CSS.
 * @param array $values Array of values.
 */
function books_map_css_selectors( &$css, $values ) {
	if ( isset( $values['css-selectors'] ) ) {
		$elements = $values['css-selectors'];
		unset( $values['css-selectors'] );

		$css[ $elements ] = $values;
	}
}

/**
 * Merge CSS values.
 *
 * @param array $css    Array of dynamic CSS.
 * @param array $values Array of values.
 */
function books_merge_value( &$css, $values ) {
	foreach ( $values as $id => $val ) {
		$css[ $id ] = $val;
	}
}

/**
 * Format of the $css array:
 * $css['media-query']['element']['property'] = value
 *
 * If no media query is required then set it to 'global'
 *
 * If we want to add multiple values for the same property then we have to make it an array like this:
 * $css[media-query][element]['property'][] = value1
 * $css[media-query][element]['property'][] = value2
 *
 * Multiple values defined as an array above will be parsed separately.
 */
function books_dynamic_css_array() {

	global $wp_version;

	$css       = [];
	$c_page_id = books()->get_page_id();

	// Site Background.
	$css['global']['html body'] = Books_Sanitize::background( books_get_settings( 'theme_background' ) );

	// Header Background.
	$css['global']['#site-header, .mobile-menu-active .navigation.mobile-menu-wrapper, .header-search .ajax-search-results-container'] = Books_Sanitize::background( books_get_settings( 'header_background' ) );

	// Logo Font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'books_logo' ) ) );
	// Footer Logo Font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'books_footer_logo' ) ) );
	// Primary Navigation font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'primary_navigation_font' ) ) );
	// Content Font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'content_font' ) ) );
	// Pagination.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'pagination_font' ) ) );
	// Archive Pages font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'archive_post_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'archive_post_excerpt_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'archive_meta_info_font' ) ) );
	// Single post title font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'single_title_font' ) ) );
	// Single post author info font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'single_author_meta_info_font' ) ) );
	// Single Page titles font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'single_page_titles_font' ) ) );
	// Single subscribe box.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'single_subscribe_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'single_subscribe_text_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'single_subscribe_input_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'single_subscribe_submit_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'single_subscribe_small_text_font' ) ) );
	// Author Box.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'authorbox_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'authorbox_author_name_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'authorbox_text_font' ) ) );
	// Subscribe widget title font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'subscribe_title_font' ) ) );
	// Fields font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'fields_font' ) ) );
	// H1 title in the content.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'h1_headline' ) ) );
	// H2 title in the content.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'h2_headline' ) ) );
	// H3 title in the content.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'h3_headline' ) ) );
	// H4 title in the content.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'h4_headline' ) ) );
	// H5 title in the content.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'h5_headline' ) ) );
	// H6 title in the content.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'h6_headline' ) ) );

	// Footer background.
	$css['global']['#site-footer'] = Books_Sanitize::background( books_get_settings( 'mts_footer_background' ) );
	// Copyrights background.
	$css['global']['.copyrights'] = Books_Sanitize::background( books_get_settings( 'mts_copyrights_background' ) );

	books_dynamic_css_skin( $css );
	books_sidebar_position( $css );
	books_header( $css );
	books_sidebar_styling( $css );
	books_home_sections( $css );
	books_post_layouts( $css );
	books_post_pagination( $css );
	books_footer( $css );
	books_copyrights( $css );
	books_pages( $css );
	books_single( $css );
	books_single_social_buttons( $css );
	books_archive_post( $css );
	books_woo( $css );
	books_misc_css( $css );

	return apply_filters( 'books_dynamic_css_array', $css );
}

/**
 * Skin CSS
 *
 * @param array $css Array of dynamic CSS.
 */
function books_dynamic_css_skin( &$css ) {

	// Colors.
	$primary_color_scheme = Books_Sanitize::color( books_get_settings( 'primary_color_scheme' ) );

	// Text Color.
	$elements = [
		'a',
		'body a:hover',
		'#primary-navigation ul li a:hover',
		'#primary-navigation ul li.current-menu-item a',
		'#primary-navigation li.menu-item-has-children > a:after',
		'.product_list_widget .amount',
		'.textwidget a',
		'#wp-calendar td#today',
		'.pnavigation2 a',
		'#tabber .inside li a:hover',
		'.fn a',
		'.sidebar .wpt_widget_content a',
		'.related-posts .title a:hover',
		'article ul li::before',
		'.single .pagination a .current:hover',
		'.postauthor h5 a:hover',
		'.related-readmore a',
		'.widget .sbutton:hover',
		'.shareit-circular.standard .share-text',
		'.breadcrumb a',
		'.rank-math-breadcrumb a',
		'.single_post .post-info a',
		'.reply a:hover',
		'#commentform input#submit:hover',
		'#mtscontact_submit:hover',
		'.latestPost .title a:hover',
		'.related-posts .title a:hover',
		'.shareit-circular.standard .fa:hover',
		'.featured-section .section-wrapper .title a:hover',
		'.recent-section .section-wrapper .title a:hover',
		'.featured-section .section-wrapper .post-meta a:hover',
		'.recent-section .section-wrapper .post-meta a:hover',
		'.blog-wrapper .title a:hover',
		'.blog-wrapper .post-meta a:hover',
		'.layout-default .latestPost .title a:hover',
		'.layout-default .latestPost .post-info a:hover',
		'.layout-default .latestPost .post-info .thecategory a',
		'.layout-grid .latestPost .title a:hover',
		'.layout-grid .latestPost .post-info a:hover',
		'#sidebar .widget li a:hover',
		'#sidebar .widget li.horizontal-small .post-title a:hover',
		'#sidebar .widget .wp_review_tab_widget_content li a:hover',
		'#sidebar .widget .wpt_widget_content li a:hover',
		'.aboutme-widget .aboutme-social a',
		'.widget #wp-subscribe input.submit:hover',
		'.toggle-menu .toggle-caret .fa',
		'.tags a:hover',
		'.tagcloud a:hover',
		'.single_post .post-info a:hover',
		'.single_post .post-info .thecategory a',
		'#sidebar .widget .wp_review_tab_widget_content ul.wp-review-tabs li.selected a',
		'#sidebar .widget .wpt_widget_content ul.wpt-tabs li.selected a',
		'#sidebar .widget.widget_categories li a:hover',
		'.related-posts .post-info a:hover',
		'#commentform p .fa',
		'.woocommerce div.product p.price, .woocommerce div.product span.price', // WooCommerce.
		'.woocommerce a.button:hover',
		'.woocommerce-page a.button:hover',
		'.woocommerce div.product form.cart .button:hover',
		'.woocommerce-page div.product form.cart .button:hover',
		'.woocommerce button.button:hover',
		'.woocommerce-page button.button:hover',
		'.woocommerce a.checkout-button.button:hover',
		'.woocommerce #payment #place_order:hover',
		'.woocommerce-page #payment #place_order:hover',
		'.woocommerce .woocommerce-widget-layered-nav-dropdown__submit:hover',
		'.woocommerce div.product .woocommerce-tabs ul.tabs li.active a',
	];

	$css['global'][ books_implode( $elements ) ]['color'] = $primary_color_scheme;
	// social svg hover color.
	$css['global']['.shareit-circular.standard svg:hover']['fill'] = $primary_color_scheme;

	// Text Color Important.
	$elements = [
		'.f-widget .widget li a:hover',
		'.widget .review_thumb_large .review-result',
		'.widget .review_thumb_large .review-total-only.large-thumb',
		'.review-total-only.small-thumb .review-result-wrapper i',
	];
	$css['global'][ books_implode( $elements ) ]['color'] = $primary_color_scheme . '!important';

	// Background Color.
	$elements = [
		'.button',
		'.widget #wp-subscribe input.submit',
		'.pace .pace-progress',
		'#mobile-menu-wrapper ul li a:hover',
		'.pagination a:hover',
		'.single .pagination a:hover .current',
		'.navigation #wpmm-megamenu .wpmm-pagination a',
		'.wpmm-megamenu-showing.wpmm-light-scheme',
		'#move-to-top:hover',
		'#tabber ul.tabs li a.selected',
		'.navigation ul .sfHover a',
		'.widget-slider .slide-caption',
		'.owl-prev:hover',
		'.owl-next:hover',
		'#wpmm-megamenu .review-total-only',
		'#searchsubmit',
		'.woocommerce .widget_price_filter .ui-slider .ui-slider-handle',
		'.woocommerce .widget_price_filter .ui-slider .ui-slider-range',
		'.woocommerce span.onsale',
		'.woocommerce .bypostauthor:after',
		'.woocommerce-account .woocommerce-MyAccount-navigation li.is-active',
		'.widget .widget_wp_review_tab .review-total-only.large-thumb',
		'.f-widget .wp_review_tab_widget_content ul.wp-review-tabs li.selected a::before',
		'.f-widget .wpt_widget_content ul.wpt-tabs li.selected a::before',
		'.pagination .nav-previous a:hover',
		'.pagination .nav-next a:hover',
		'.single_post article .post ul li::before',
		'.single-subscribe .widget #wp-subscribe input.submit',
		'.instagram-button a',
		'.tagcloud a',
		'#commentform input#submit',
		'#mtscontact_submit',
		'.widget .wp_review_tab_widget_content ul.wp-review-tabs li.selected a::before',
		'.widget .wpt_widget_content ul.wpt-tabs li.selected a::before',
		'.woocommerce a.button', // WooCommerce.
		'.woocommerce-page a.button',
		'.woocommerce button.button',
		'.woocommerce-page button.button',
		'.woocommerce input.button',
		'.woocommerce-page input.button',
		'.woocommerce #respond input#submit',
		'.woocommerce-page #respond input#submit',
		'.woocommerce #content input.button',
		'.woocommerce-page #content input.button',
		'.woocommerce a.button',
		'.woocommerce-page a.button',
		'.woocommerce button.button',
		'.woocommerce-page button.button',
		'.woocommerce input.button',
		'.woocommerce-page input.button',
		'.woocommerce #respond input#submit',
		'.woocommerce-page #respond input#submit',
		'.woocommerce #content input.button',
		'.woocommerce-page #content input.button',
		'.woocommerce #respond input#submit.alt.disabled',
		'.woocommerce #respond input#submit.alt:disabled[disabled]',
		'.woocommerce a.button.alt:disabled',
		'.woocommerce a.button.alt:disabled[disabled]',
		'.woocommerce button.button.alt.disabled',
		'.woocommerce button.button.alt:disabled[disabled]',
		'#add_payment_method .wc-proceed-to-checkout a.checkout-button',
		'.woocommerce-cart .wc-proceed-to-checkout a.checkout-button',
		'.woocommerce-checkout .wc-proceed-to-checkout a.checkout-button',
		'.woocommerce a.button.alt',
		'.woocommerce button.button.alt',
		'.woocommerce input.button.alt',
		'.woocommerce .woocommerce-widget-layered-nav-dropdown__submit',
		'.woocommerce .widget.widget_product_search button[type="submit"]',
		'.woocommerce-page .widget.widget_product_search button[type="submit"]',
		'.woocommerce div.product form.cart .button',
	];

	$css['global'][ books_implode( $elements ) ]['background-color'] = $primary_color_scheme;

	// Border Color.
	$elements = [
		'.page-numbers.current',
		'.tagcloud a',
		'.tagcloud a:hover',
		'.instagram-button a',
		'.instagram-button a:hover',
		'.widget #wp-subscribe input.submit',
		'.widget #wp-subscribe input.submit:hover',
		'#commentform input#submit',
		'#mtscontact_submit',
		'#commentform input#submit:hover',
		'#mtscontact_submit:hover',
		'.woocommerce a.button', // WooCommerce.
		'.woocommerce-page a.button',
		'.woocommerce a.button:hover',
		'.woocommerce-page a.button:hover',
		'.woocommerce div.product form.cart .button',
		'.woocommerce-page div.product form.cart .button',
		'.woocommerce div.product form.cart .button:hover',
		'.woocommerce-page div.product form.cart .button:hover',
		'.woocommerce #respond input#submit',
		'.woocommerce-page #respond input#submit',
		'.woocommerce button.button:hover',
		'.woocommerce-page button.button:hover',
		'.woocommerce .cart .button',
		'.woocommerce .cart input.button',
		'.woocommerce #payment #place_order',
		'.woocommerce-page #payment #place_order',
		'.woocommerce .woocommerce-widget-layered-nav-dropdown__submit:hover',
		'.woocommerce div.product .woocommerce-tabs ul.tabs::before',
	];

	$css['global'][ books_implode( $elements ) ]['border-color'] = $primary_color_scheme;

}

/**
 * Sidebar Position
 *
 * @param array $css Array of dynamic CSS.
 */
function books_sidebar_position( &$css ) {

	// Sidebar position.
	$sidebar_position = books_get_settings( 'theme_layout' );

	$sidebar_metabox_location = '';
	if ( is_page() || is_single() ) {
		$sidebar_metabox_location = get_post_meta( get_the_ID(), '_mts_sidebar_location', true );
	}

	if ( 'right' !== $sidebar_metabox_location && ( 'sclayout' === $sidebar_position || 'left' === $sidebar_metabox_location ) ) {
		$css['global']['.article']['float']                = 'right';
		$css['global']['.sidebar.c-4-12']['float']         = 'left';
		$css['global']['.sidebar.c-4-12']['padding-right'] = 0;

		if ( null !== books_get_settings( 'mts_social_button_position' ) && 'floating' === books_get_settings( 'mts_social_button_position' ) ) {

			$shareit_class = [
				'.sclayout .shareit.shareit-default.floating',
				'.sclayout .shareit.standard.floating',
				'.sclayout .shareit.shareit-circular.floating',
			];

			$css['global'][ books_implode( $shareit_class ) ]['left']  = 'auto';
			$css['global'][ books_implode( $shareit_class ) ]['right'] = '0px';

			$css['global'][ books_implode( $shareit_class ) ]['margin-left']  = '0px';
			$css['global'][ books_implode( $shareit_class ) ]['margin-right'] = books_px2rem( '100px' );
		}
	}
	if ( 'mts_nosidebar' === mts_custom_sidebar() ) {
		$css['global']['.single-full-header']['width'] = '100%';
	}
}

/**
 * Header
 *
 * @param array $css Array of dynamic CSS.
 */
function books_header( &$css ) {

	$header_icons  = books_get_settings( 'header_social_icons' );
	$header_layout = books_get_settings( 'header_styles' );

	// Header.
	books_merge_value( $css['global']['#header'], Books_Sanitize::margin( books_get_settings( 'header_margin' ) ) );
	books_merge_value( $css['global']['#header'], Books_Sanitize::padding( books_get_settings( 'header_padding' ) ) );

	// Header Border.
	$header_border = Books_Sanitize::border( books_get_settings( 'header_border' ) );
	$css['global']['#header'][ $header_border['direction'] ] = $header_border ['value'];

	// Nav styling.
	// Nav color.
	$css['global']['#primary-navigation .navigation ul ul a, a#pull']['color'] = Books_Sanitize::color( books_get_settings( 'main_navigation_dropdown_color' ) );

	$css['global']['#primary-navigation .navigation ul ul a:hover']['color'] = Books_Sanitize::color( books_get_settings( 'main_navigation_dropdown_hover_color' ) );

	// Nav Dropdown Background.
	$css['global']['.navigation ul ul']['background-color'] = Books_Sanitize::color( books_get_settings( 'main_nav_dropdown_bg' ) );

	// Nav margin and Padding.
	books_merge_value( $css['global']['#primary-navigation'], Books_Sanitize::margin( books_get_settings( 'header_nav_margin' ) ) );
	books_merge_value( $css['global']['#primary-navigation'], Books_Sanitize::padding( books_get_settings( 'header_nav_padding' ) ) );

	// Header layout 2.
	if ( 'header-layout2' === $header_layout ) {

		$css['global']['.header-layout2 #primary-navigation']['background-color'] = Books_Sanitize::color( books_get_settings( 'main_nav_bg_color' ) );

		books_merge_value( $css['global']['#primary-navigation'], Books_Sanitize::margin( books_get_settings( 'header2_nav_margin' ) ) );

		books_merge_value( $css['global']['#primary-navigation'], Books_Sanitize::padding( books_get_settings( 'header2_nav_padding' ) ) );

	}

	// Social icons.
	if ( $header_icons && is_array( $header_icons ) ) {

		foreach ( $header_icons as $header_icon ) {

			$css['global'][ '.header-social-icons a.header-' . $header_icon['header_icon'] ]['background-color'] = Books_Sanitize::color( $header_icon['header_icon_bgcolor'] );

			$css['global'][ '.header-social-icons a.header-' . $header_icon['header_icon'] . ':hover' ]['background-color'] = Books_Sanitize::color( $header_icon['header_icon_hover_bgcolor'] );

			$css['global'][ '.header-social-icons a.header-' . $header_icon['header_icon'] ]['color'] = Books_Sanitize::color( $header_icon['header_icon_color'] );

			$css['global'][ '.header-social-icons a.header-' . $header_icon['header_icon'] . ':hover' ]['color'] = Books_Sanitize::color( $header_icon['header_icon_hover_color'] );

			books_merge_value( $css['global'][ '.header-social-icons a.header-' . $header_icon['header_icon'] ], Books_Sanitize::margin( $header_icon['header_icon_margin'] ) );

			books_merge_value( $css['global'][ '.header-social-icons a.header-' . $header_icon['header_icon'] ], Books_Sanitize::padding( $header_icon['header_icon_padding'] ) );

			$css['global'][ '.header-social-icons a.header-' . $header_icon['header_icon'] ]['border-radius'] = Books_Sanitize::size( $header_icon['header_icon_border_radius'] . 'px' );

		}
	}

	// Header Ad.
	books_merge_value( $css['global']['.widget-header'], Books_Sanitize::margin( books_get_settings( 'mts_header_adcode_margin' ) ) );

	// Ad-Blocker.
	$css['global']['.navigation-banner']['background'] = Books_Sanitize::color( books_get_settings( 'navigation_ad_background' ) );

	// Floating Navigation Responsive.
	if ( ! books_get_settings( 'sticky_responsive_nav' ) ) {
		$css['@media screen and (max-width: 865px)']['#catcher']['height']                           = '0px!important';
		$css['@media screen and (max-width: 865px)']['.sticky-navigation-active']['position']        = 'relative!important';
		$css['@media screen and (max-width: 865px)']['.sticky-navigation-active']['top']             = '0px!important';
		$css['@media screen and (max-width: 865px)']['.admin-bar .sticky-navigation-active ']['top'] = '0px!important';
	}
}

/**
 * Social Share Styling
 *
 * @param array $css Array of dynamic CSS.
 */
function books_single_social_buttons( &$css ) {

	// Social share.
	$css['global']['.shareit.floating'] = Books_Sanitize::background( books_get_settings( 'social_styling_background' ) );
	books_merge_value( $css['global']['.shareit.floating'], Books_Sanitize::margin( books_get_settings( 'social_styling_margin' ) ) );

	// Social share border.
	$social_border = Books_Sanitize::border( books_get_settings( 'social_styling_border' ) );
	$css['global']['.shareit.floating'][ $social_border ['direction'] ] = $social_border ['value'];

	$social_button_layout   = books_get_settings( 'social_button_layout' );
	$social_button_position = books_get_settings( 'social_floating_button_position' );
	if ( 'default' === $social_button_layout ) {
		$share_class = '.shareit.shareit-' . $social_button_layout . '.floating';
	} else {
		$share_class = '.shareit.' . $social_button_layout . '.floating';
	}
	if ( ! empty( $social_button_position ) && is_array( $social_button_position ) ) {
		foreach ( $social_button_position as $key => $position ) {
			$css['global'][ $share_class ][ $key ] = $position;
		}
	}
}

/**
 * Sidebar styling
 *
 * @param array $css Array of dynamic CSS.
 */
function books_sidebar_styling( &$css ) {

	// Sidebar title.
	$sidebar_title = '#sidebar .widget h3';

	$css['global'][ $sidebar_title ] = Books_Sanitize::background( books_get_settings( 'sidebar_title_styling_background' ) );
	books_merge_value( $css['global'][ $sidebar_title ], Books_Sanitize::padding( books_get_settings( 'sidebar_title_styling_padding' ) ) );
	books_merge_value( $css['global'][ $sidebar_title ], Books_Sanitize::margin( books_get_settings( 'sidebar_title_styling_margin' ) ) );

	// Sidebar Title border.
	$sidebar_title_border = Books_Sanitize::border( books_get_settings( 'widget_title_border' ) );
	$css['global'][ $sidebar_title ][ $sidebar_title_border['direction'] ] = $sidebar_title_border['value'];

	// Sidebar.
	books_merge_value( $css['global']['.sidebar .widget'], Books_Sanitize::background( books_get_settings( 'sidebar_styling_background' ) ) );
	books_merge_value( $css['global']['.sidebar .widget'], Books_Sanitize::margin( books_get_settings( 'sidebar_styling_margin' ) ) );
	books_merge_value( $css['global']['.sidebar .widget'], Books_Sanitize::padding( books_get_settings( 'sidebar_styling_padding' ) ) );

	// Sidebar border.
	$sidebar_border = Books_Sanitize::border( books_get_settings( 'sidebar_styling_border' ) );
	$css['global']['.sidebar .widget'][ $sidebar_border['direction'] ] = $sidebar_border['value'];

	// Border Radius.
	$css['global']['.sidebar .widget']['border-radius'] = Books_Sanitize::size( books_get_settings( 'sidebar_styling_border_radius' ) . 'px' );

	// Box shadow.
	if ( ! books_get_settings( 'sidebar_styling_shadow' ) ) {
		$css['global']['.sidebar .widget']['box-shadow'] = 'none';
	}

	// Typography.
	// Sidebar widget title font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'sidebar_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'sidebar_buttons_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'submit_button_font' ) ) );
	// Sidebar widget font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'mts_widget_links' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'default_widget_links' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'sidebar_url_bigthumb' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'sidebar_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'sidebar_font_bigthumb' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'sidebar_postinfo_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'sidebar_postinfo_font_bigthumb' ) ) );

	/**
	 * Search widget.
	 */
	books_merge_value( $css['global']['.widget.widget_search'], Books_Sanitize::padding( books_get_settings( 'search_widget_padding' ) ) );

	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'search_fields_font' ) ) );

	/**
	 * Tab widgets.
	 */
	books_merge_value( $css['global']['.widget.widget_wp_review_tab, .widget.widget_wpt'], Books_Sanitize::padding( books_get_settings( 'tab_widgets_padding' ) ) );

	// Tab widget title font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'tabs_title_font' ) ) );

	/**
	 * About Us widget.
	 */
	$about_paddings = Books_Sanitize::padding( books_get_settings( 'sidebar_styling_padding' ) );
	$padding        = books_get_total_padding( $about_paddings );

	$css['global']['.aboutme-widget .aboutme-image img']['width']       = '-' . $padding / 2 . 'px';
	$css['global']['.aboutme-widget .aboutme-image img']['max-width']   = '-' . $padding / 2 . 'px';
	$css['global']['.aboutme-widget .aboutme-image img']['margin-left'] = '-' . $padding / 2 . 'px';

	/**
	 * Instagram widget.
	 */
	$insta_paddings = Books_Sanitize::padding( books_get_settings( 'instagram_widget_padding' ) );
	$padding        = books_get_total_padding( $insta_paddings );

	$css['global']['.instagram-widget']['margin-left']  = '-' . $padding / 2 . 'px';
	$css['global']['.instagram-widget']['margin-right'] = '-' . $padding / 2 . 'px';

	books_merge_value( $css['global']['.widget.instagram-widget-wrapper'], Books_Sanitize::padding( books_get_settings( 'instagram_widget_padding' ) ) );

	// Tags.
	$css['global']['.tagcloud a, .widget .wpt_widget_content #tags-tab-content ul li a']['background-color'] = Books_Sanitize::color( books_get_settings( 'tags_bgcolor' ) );

	/**
	 * Footer widget settings.
	 */
	books_merge_value( $css['global']['.f-widget .widget'], Books_Sanitize::background( books_get_settings( 'f_widget_background' ) ) );
	books_merge_value( $css['global']['.f-widget .widget'], Books_Sanitize::margin( books_get_settings( 'f_widget_margin' ) ) );
	books_merge_value( $css['global']['.f-widget .widget'], Books_Sanitize::padding( books_get_settings( 'f_widget_padding' ) ) );

	// Border.
	$f_widget_border = Books_Sanitize::border( books_get_settings( 'f_widget_border' ) );
	$css['global']['.f-widget .widget'][ $f_widget_border['direction'] ] = $f_widget_border['value'];

	// Border Radius.
	$css['global']['.f-widget .widget']['border-radius'] = Books_Sanitize::size( books_get_settings( 'f_widget_border_radius' ) . 'px' );

	// Box shadow.
	if ( ! books_get_settings( 'f_widget_shadow' ) ) {
		$css['global']['.f-widget .widget']['box-shadow'] = 'none';
	}

	// Typography.
	// Footer widget title font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'footer_books_logo' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'top_footer_title_font' ) ) );
	// Footer link font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'top_footer_link_font' ) ) );
	// Footer MTS link font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'mts_footer_widget_links' ) ) );
	// Footer widget font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'top_footer_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'footer_metainfo_font' ) ) );
	// Copyrights section font.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'copyrights_font' ) ) );

}

/**
 * HomePage Sections
 *
 * @param array $css Array of dynamic CSS.
 */
function books_home_sections( &$css ) {

	$categories_list      = books_get_settings( 'categories_list' ); // Slider Section.
	$home_categories_list = books_get_settings( 'home_categories_list' ); // Categories Section.

	/**
	 * Slider Section.
	 */
	books_merge_value( $css['global']['.slider-section'], Books_Sanitize::background( books_get_settings( 'slider_background' ) ) );

	if ( $categories_list && is_array( $categories_list ) ) {

		foreach ( $categories_list as $category ) {

			$class    = '.slider-section li.product_cat-' . $category['product_category'];
			$g_deg    = $category['category_gradient_deg'];
			$g_top    = $category['category_gradient_top'];
			$g_bottom = $category['category_gradient_bottom'];
			$gradient = join( ', ', [ $g_deg, $g_top, $g_bottom ] );

			$css['global'][ $class ]['background'] = 'linear-gradient(' . $gradient . ');';

		}

	}

	// Border Radius.
	$css['global']['.slider-section li, .slider-section .featured-thumbnail img']['border-radius'] = Books_Sanitize::size( books_get_settings( 'slider_items_border_radius' ) . 'px' );

	// Margin and Padding.
	books_merge_value( $css['global']['.slider-section li'], Books_Sanitize::padding( books_get_settings( 'slider_items_padding' ) ) );
	books_merge_value( $css['global']['.slider-section'], Books_Sanitize::margin( books_get_settings( 'slider_margin' ) ) );
	books_merge_value( $css['global']['.slider-section'], Books_Sanitize::padding( books_get_settings( 'slider_padding' ) ) );

	// Typography.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'slider_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'slider_meta_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'slider_excerpt' ) ) );

	/**
	 * Categories Section.
	 */
	books_merge_value( $css['global']['.categories-section'], Books_Sanitize::background( books_get_settings( 'categories_background' ) ) );

	if ( $home_categories_list && is_array( $home_categories_list ) ) {

		$count = 0;

		foreach ( $home_categories_list as $category ) {

			$class = '.categories-section a.cat-item-' . ( ++$count );

			$css['global'][ $class ]['background'] = Books_Sanitize::color( $category['home_category_bg'] );

			$css['global'][ $class . ' .cat-name' ]['color'] = Books_Sanitize::color( $category['home_category_title_color'] );

			$css['global'][ $class . ':hover .cat-name' ]['color'] = Books_Sanitize::color( $category['home_category_icon_bg'] );

			$css['global'][ $class . ' .icon' ]['background'] = Books_Sanitize::color( $category['home_category_icon_bg'] );

			$css['global'][ $class . ' .icon' ]['color'] = Books_Sanitize::color( $category['home_category_icon_color'] );

			$css['global'][ $class . ' .post-count' ]['color'] = Books_Sanitize::color( $category['category_count_color'] );
		}
	}

	// Typography.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'home_categories_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'home_categories_count' ) ) );

	// Border Radius.
	$css['global']['.categories-section a']['border-radius'] = Books_Sanitize::size( books_get_settings( 'home_categories_border_radius' ) . 'px' );

	// Box shadow.
	if ( ! books_get_settings( 'home_categories_shadow' ) ) {
		$css['global']['.categories-section a']['box-shadow'] = 'none';
	}

	// Margin and Padding.
	books_merge_value( $css['global']['.categories-section'], Books_Sanitize::margin( books_get_settings( 'home_categories_margin' ) ) );
	books_merge_value( $css['global']['.categories-section'], Books_Sanitize::padding( books_get_settings( 'home_categories_padding' ) ) );

	/**
	 * Featured Section.
	 */
	books_merge_value( $css['global']['.featured-section'], Books_Sanitize::background( books_get_settings( 'featured_background' ) ) );

	// Typography.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'featured_section_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'featured_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'featured_meta_font' ) ) );

	// Border Radius.
	$css['global']['.featured-section .featured-thumbnail img, .section-post-thumb:hover .overlay']['border-radius'] = Books_Sanitize::size( books_get_settings( 'featured_thumb_border_radius' ) . 'px' );

	// Box shadow.
	if ( ! books_get_settings( 'featured_thumb_shadow' ) ) {
		$css['global']['.featured-section .featured-thumbnail img']['box-shadow'] = 'none';
	}

	// Margin and Padding.
	books_merge_value( $css['global']['.featured-section'], Books_Sanitize::margin( books_get_settings( 'featured_margin' ) ) );
	books_merge_value( $css['global']['.featured-section'], Books_Sanitize::padding( books_get_settings( 'featured_padding' ) ) );

	/**
	 * Recent Section.
	 */
	books_merge_value( $css['global']['.recent-section'], Books_Sanitize::background( books_get_settings( 'recent_background' ) ) );

	// Typography.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'recent_section_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'recent_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'recent_meta_font' ) ) );

	// Border Radius.
	$css['global']['.recent-section .featured-thumbnail img']['border-radius'] = Books_Sanitize::size( books_get_settings( 'recent_thumb_border_radius' ) . 'px' );

	// Box shadow.
	if ( ! books_get_settings( 'recent_thumb_shadow' ) ) {
		$css['global']['.recent-section .featured-thumbnail img']['box-shadow'] = 'none';
	}

	// Margin and Padding.
	books_merge_value( $css['global']['.recent-section'], Books_Sanitize::margin( books_get_settings( 'recent_margin' ) ) );
	books_merge_value( $css['global']['.recent-section'], Books_Sanitize::padding( books_get_settings( 'recent_padding' ) ) );

	/**
	 * Blog Section.
	 */
	books_merge_value( $css['global']['.blog-section'], Books_Sanitize::background( books_get_settings( 'blog_background' ) ) );

	// Typography.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'blog_section_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'blog_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'blog_meta_info_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'blog_content_font' ) ) );

	// Margin and Padding.
	books_merge_value( $css['global']['.blog-section'], Books_Sanitize::margin( books_get_settings( 'blog_margin' ) ) );
	books_merge_value( $css['global']['.blog-section'], Books_Sanitize::padding( books_get_settings( 'blog_padding' ) ) );

	/**
	 * Subscribe Section.
	 */
	$subscribe_class = '.subscribe-section .subscribe-wrapper';
	if ( 'full' === books_get_settings( 'subscribe_section_width' ) ) {
		$subscribe_class = '.subscribe-section';
	}
	books_merge_value( $css['global'][ $subscribe_class ], Books_Sanitize::background( books_get_settings( 'subscribe_background' ) ) );

	// Typography.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'subscribe_heading_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'subscribe_heading_desc_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'subscribe_fields_font' ) ) );

	// Border Radius.
	$css['global'][ $subscribe_class ]['border-radius'] = Books_Sanitize::size( books_get_settings( 'subscribe_border_radius' ) . 'px' );

	// Margin and Padding.
	books_merge_value( $css['global'][ $subscribe_class ], Books_Sanitize::margin( books_get_settings( 'subscribe_margin' ) ) );
	books_merge_value( $css['global'][ $subscribe_class ], Books_Sanitize::padding( books_get_settings( 'subscribe_padding' ) ) );

	/**
	 * Custom HTML.
	 */
	// Margin and Padding.
	books_merge_value( $css['global']['.html-section'], Books_Sanitize::margin( books_get_settings( 'html_margin' ) ) );
	books_merge_value( $css['global']['.html-section'], Books_Sanitize::padding( books_get_settings( 'html_padding' ) ) );

}

/**
 * Layout CSS
 *
 * @param array $css Array of dynamic CSS.
 */
function books_post_layouts( &$css ) {

	// Post Layouts.
	$features = books_get_settings( 'featured_categories' );

	if ( $features && is_array( $features ) ) {

		foreach ( $features as $feature ) {

			if ( ! isset( $feature['unique_id'] ) ) {
				continue;
			}

			$category     = $feature['mts_featured_category'];
			$posts_layout = isset( $feature['blog_layout'] ) ? $feature['blog_layout'] : 'layout-default';
			$unique_id    = $feature['unique_id'];

			if ( 'layout-default' === $posts_layout ) :
				$posts_layout = 'default';
			endif;

			// Section title align.
			$post_title_align = books_get_settings( 'post_title_alignment_' . $unique_id );

			// Post area.
			$cat_class = 'cat-latest';
			if ( 'latest' !== $category ) {
				$category  = get_term_by( 'slug', $category, 'category' );
				$cat_class = sanitize_key( $category->name );
			}

			$title_class = '.title-container.title-id-' . $unique_id . ' h3';

			$css['global'][ $title_class ] = Books_Sanitize::background( books_get_settings( 'featured_category_title_background_' . $unique_id ) );
			books_merge_value( $css['global'][ $title_class ], Books_Sanitize::margin( books_get_settings( 'featured_category_title_margin_' . $unique_id ) ) );
			books_merge_value( $css['global'][ $title_class ], Books_Sanitize::padding( books_get_settings( 'featured_category_title_padding_' . $unique_id ) ) );
			// Section title border.
			$post_title_border = Books_Sanitize::border( books_get_settings( 'post_title_border_' . $unique_id ) );
			$css['global'][ $title_class ][ $post_title_border['direction'] ] = $post_title_border['value'];

			// Title alignment.
			$align_class = '.title-container.title-id-' . $unique_id;
			if ( 'center' === $post_title_align ) :
				$css['global'][ $align_class ]['text-align'] = 'center';
			elseif ( 'right' === $post_title_align ) :
				$css['global'][ $align_class ]['text-align']              = 'right';
				$css['global'][ $align_class . ' + .view-more' ]['left']  = '0';
				$css['global'][ $align_class . ' + .view-more' ]['right'] = 'auto';
			endif;

			books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'featured_category_title_font_' . $unique_id ) ) );

			if ( 'layout-default' === $posts_layout ) {
				$post_class = '.layout-' . $unique_id . '.default-container';
			} else {
				$post_class = '.layout-' . $unique_id;
			}

			// Post border.
			$post_border = Books_Sanitize::border( books_get_settings( 'post_border_' . $unique_id ) );
			$css['global'][ $post_class ][ $post_border['direction'] ] = $post_border['value'];

			/**
			 * Meta info
			 */
			books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'meta_info_font_' . $unique_id ) ) );

			books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'post_title_font_' . $unique_id ) ) );
			books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'post_excerpt_font_' . $unique_id ) ) );

			// All Posts Margin and Padding.
			books_merge_value( $css['global'][ $post_class ], Books_Sanitize::margin( books_get_settings( 'post_margin_' . $unique_id ) ) );
			books_merge_value( $css['global'][ $post_class ], Books_Sanitize::padding( books_get_settings( 'post_padding_' . $unique_id ) ) );

			if ( 'layout-full' === $posts_layout ) {
				$css['global'][ '.layout-' . $unique_id . ' .wrapper' ]['background-color'] = Books_Sanitize::color( books_get_settings( 'full_post_overlay_' . $unique_id ) );
			}

		}
	}
}

/**
 * Pagination
 *
 * @param array $css Array of dynamic CSS.
 */
function books_post_pagination( &$css ) {

	// Pagination Active class.
	$pagination_class_active = [
		'.pace .pace-progress',
		'.page-numbers.current',
		'.pagination a:hover',
	];
	$pagination_class        = [
		'.pagination a',
		'.single .pagination > .current .currenttext',
		'.pagination .page-numbers.dots',
	];
	books_merge_value( $css['global'][ books_implode( array_merge( $pagination_class_active, $pagination_class ) ) ], Books_Sanitize::margin( books_get_settings( 'mts_pagenavigation_margin' ) ) );
	if ( '2' !== books_get_settings( 'mts_pagenavigation_type' ) ) {

		$css['global'][ books_implode( $pagination_class ) ] = Books_Sanitize::background( books_get_settings( 'mts_pagenavigation_bgcolor' ) );

		$css['global'][ books_implode( $pagination_class_active ) ] = Books_Sanitize::background( books_get_settings( 'mts_pagenavigation_hover_bgcolor' ) );

		$css['global']['.page-numbers.current'] = Books_Sanitize::background( books_get_settings( 'mts_pagenavigation_active_bgcolor' ) );

		$css['global'][ books_implode( $pagination_class ) ]['color'] = Books_Sanitize::color( books_get_settings( 'mts_pagenavigation_color' ) );

		$css['global'][ books_implode( $pagination_class_active ) ]['color'] = Books_Sanitize::color( books_get_settings( 'mts_pagenavigation_hover_color' ) );

		books_merge_value( $css['global'][ books_implode( array_merge( $pagination_class_active, $pagination_class ) ) ], Books_Sanitize::padding( books_get_settings( 'mts_pagenavigation_padding' ) ) );

	} else {

		$css['global']['#load-posts a'] = Books_Sanitize::background( books_get_settings( 'load_more_bgcolor' ) );

		$css['global']['#load-posts a:hover'] = Books_Sanitize::background( books_get_settings( 'load_more_hover_bgcolor' ) );

		$css['global']['#load-posts a']['color'] = Books_Sanitize::color( books_get_settings( 'load_more_color' ) );

		$css['global']['#load-posts a:hover']['color'] = Books_Sanitize::color( books_get_settings( 'load_more_hover_color' ) );

		books_merge_value( $css['global']['#load-posts a'], Books_Sanitize::padding( books_get_settings( 'load_more_padding' ) ) );

	}

	$border_radius_class   = $pagination_class;
	$border_radius_class[] = '#load-posts a';

	// Border radius.
	$css['global'][ books_implode( $border_radius_class ) ]['border-radius'] = Books_Sanitize::size( books_get_settings( 'mts_pagenavigation_border_radius' ) . 'px' );

	$css['global'][ books_implode( $pagination_class_active ) ]['border-radius'] = Books_Sanitize::size( books_get_settings( 'mts_pagenavigation_border_radius' ) . 'px' );

	// Pagination border.
	$pagination_border = Books_Sanitize::border( books_get_settings( 'pagenavigation_border' ) );
	$css['global'][ books_implode( array_merge( $pagination_class_active, $border_radius_class ) ) ][ $pagination_border ['direction'] ] = $pagination_border ['value'];

	// Pagination Alignment.
	$pagenavigation_align = books_get_settings( 'pagenavigation_alignment' );
	if ( 'left' === $pagenavigation_align ) :
		$css['global']['.pagination']['text-align'] = 'left';
	elseif ( 'right' === $pagenavigation_align ) :
		$css['global']['.pagination']['text-align'] = 'right';
	endif;

	// Load more Alignment.
	$load_more_align = books_get_settings( 'load_more_alignment' );
	if ( 'left' === $load_more_align ) :
		$css['global']['#load-posts']['text-align'] = 'left';
	elseif ( 'right' === $load_more_align ) :
		$css['global']['#load-posts']['text-align'] = 'right';
	elseif ( 'full' === $load_more_align ) :
		$css['global']['#load-posts a']['width'] = '100%';
	endif;
}

/**
 * Single
 *
 * @param array $css Array of dynamic CSS.
 */
function books_pages( &$css ) {

	$page_classes  = [
		'.page .article',
		'.error404 .article',
		'.woocommerce-page .single_post_wrap.wc_page .article',
	];

	$css['global'][ books_implode( $page_classes ) ] = Books_Sanitize::background( books_get_settings( 'pages_background' ) );

	// Margin and Padding.
	books_merge_value( $css['global'][ books_implode( $page_classes ) ], Books_Sanitize::margin( books_get_settings( 'pages_margin' ) ) );
	books_merge_value( $css['global'][ books_implode( $page_classes ) ], Books_Sanitize::padding( books_get_settings( 'pages_padding' ) ) );

	// Border.
	$single_border = Books_Sanitize::border( books_get_settings( 'pages_border' ) );
	$css['global'][ books_implode( $page_classes ) ][ $single_border ['direction'] ] = $single_border ['value'];

	// Border Radius.
	$css['global'][ books_implode( $page_classes ) ]['border-radius'] = Books_Sanitize::size( books_get_settings( 'pages_border_radius' ) . 'px' );

	// Box shadow.
	if ( ! books_get_settings( 'pages_shadow' ) ) {
		$css['global'][ books_implode( $page_classes ) ]['box-shadow'] = 'none';
	}

}

/**
 * Single
 *
 * @param array $css Array of dynamic CSS.
 */
function books_single( &$css ) {

	// Single.
	$css['global']['.article .single_post'] = Books_Sanitize::background( books_get_settings( 'single_background' ) );

	/**
	 * Featured Image.
	 */
	$s_paddings = Books_Sanitize::padding( books_get_settings( 'mts_single_styling_padding' ) );
	$padding    = books_get_total_padding( $s_paddings );

	$css['global']['.single_post .single-featured-wrap']['margin-left']  = '-' . $padding / 2 . 'px';
	$css['global']['.single_post .single-featured-wrap']['margin-right'] = '-' . $padding / 2 . 'px';
	$css['global']['.single_post .single-featured-wrap']['margin-top']   = '-' . $s_paddings['padding-top'];

	// Margin and Padding.
	books_merge_value( $css['global']['.single.single_post_wrap'], Books_Sanitize::margin( books_get_settings( 'mts_single_styling_margin' ) ) );
	books_merge_value( $css['global']['.article .single_post'], $s_paddings );

	// Single border.
	$single_border = Books_Sanitize::border( books_get_settings( 'single_styling_border' ) );
	$css['global']['.article .single_post'][ $single_border ['direction'] ] = $single_border ['value'];

	// Border Radius.
	$css['global']['.article .single_post']['border-radius'] = Books_Sanitize::size( books_get_settings( 'single_styling_border_radius' ) . 'px' );

	// Box shadow.
	if ( ! books_get_settings( 'single_styling_shadow' ) ) {
		$css['global']['.article .single_post']['box-shadow'] = 'none';
	}

	// Typography.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'mts_single_meta_info_font' ) ) );

	/**
	 * Breadcrumbs.
	 */
	books_merge_value( $css['global']['.breadcrumb'], Books_Sanitize::background( books_get_settings( 'breadcrumb_background' ) ) );
	books_merge_value( $css['global']['.breadcrumb'], Books_Sanitize::margin( books_get_settings( 'breadcrumb_margin' ) ) );
	books_merge_value( $css['global']['.breadcrumb'], Books_Sanitize::padding( books_get_settings( 'breadcrumb_padding' ) ) );

	// Border Radius.
	$css['global']['.breadcrumb']['border-radius'] = Books_Sanitize::size( books_get_settings( 'breadcrumb_border_radius' ) . 'px' );

	// Box shadow.
	if ( ! books_get_settings( 'breadcrumb_shadow' ) ) {
		$css['global']['.breadcrumb']['box-shadow'] = 'none';
	}

	// Typography.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'breadcrumb_font' ) ) );

	/**
	 * Tag section.
	 */
	$css['global']['.tags::before']['width']       = 'calc(100% + ' . $padding . 'px)';
	$css['global']['.tags::before']['margin-left'] = '-' . $padding / 2 . 'px';

	/**
	 * Social Icons section.
	 */
	$css['global']['.share-wrap::before']['width']       = 'calc(100% + ' . $padding . 'px)';
	$css['global']['.share-wrap::before']['margin-left'] = '-' . $padding / 2 . 'px';

	$css['@media screen and (max-width: 1330px)']['.shareit.floating::before']['width']       = 'calc(100% + ' . $padding . 'px)';
	$css['@media screen and (max-width: 1330px)']['.shareit.floating::before']['margin-left'] = '-' . $padding / 2 . 'px';

	// Related Posts.
	$css['global']['.related-posts'] = Books_Sanitize::background( books_get_settings( 'related_posts_background' ) );

	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'related_posts_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'related_posts_meta_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'related_posts_excerpt_font' ) ) );

	books_merge_value( $css['global']['.related-posts'], Books_Sanitize::margin( books_get_settings( 'related_posts_margin' ) ) );
	books_merge_value( $css['global']['.related-posts'], Books_Sanitize::padding( books_get_settings( 'related_posts_padding' ) ) );

	$related_posts_border = Books_Sanitize::border( books_get_settings( 'related_posts_border' ) );
	$css['global']['.related-posts'][ $related_posts_border ['direction'] ] = $related_posts_border ['value'];

	// Related Posts articles.
	$css['global']['.related-posts article'] = Books_Sanitize::background( books_get_settings( 'related_article_background' ) );

	books_merge_value( $css['global']['.related-posts article'], Books_Sanitize::padding( books_get_settings( 'related_article_padding' ) ) );

	books_merge_value( $css['global']['.related-posts article .wrapper'], Books_Sanitize::padding( books_get_settings( 'related_article_text_padding' ) ) );

	// Border Radius.
	$css['global']['.related-posts #featured-thumbnail img']['border-radius'] = Books_Sanitize::size( books_get_settings( 'related_post_image_border_radius' ) . 'px' );

	// Box Shadow.
	if ( ! books_get_settings( 'related_article_shadow' ) ) {
		$css['global']['.related-posts article']['box-shadow'] = 'none';
	}

	// Subscribe Box.
	$css['global']['.single-subscribe .widget.wp_subscribe'] = Books_Sanitize::background( books_get_settings( 'single_subscribe_background' ) );
	books_merge_value( $css['global']['.single-subscribe .widget.wp_subscribe'], Books_Sanitize::margin( books_get_settings( 'single_subscribe_margin' ) ) );
	books_merge_value( $css['global']['.single-subscribe .widget.wp_subscribe'], Books_Sanitize::padding( books_get_settings( 'single_subscribe_padding' ) ) );
	$css['global']['.single-subscribe .widget.wp_subscribe']['border-radius'] = Books_Sanitize::size( books_get_settings( 'single_subscribe_border_radius' ) . 'px' );

	// Box Shadow.
	if ( ! books_get_settings( 'single_subscribe_shadow' ) ) {
		$css['global']['.single-subscribe .widget.wp_subscribe']['box-shadow'] = 'none';
	}

	// Subscribe border.
	$subscribe_border = Books_Sanitize::border( books_get_settings( 'single_subscribe_border' ) );
	$css['global']['.single-subscribe .widget.wp_subscribe'][ $subscribe_border ['direction'] ] = $subscribe_border ['value'];

	// Subscribe Box Input fields.
	$subscribe_input_class = [
		'.single-subscribe #wp-subscribe input.email-field',
		'.single-subscribe #wp-subscribe input.name-field',
	];
	$css['global'][ books_implode( $subscribe_input_class ) ]['height'] = Books_Sanitize::size( books_get_settings( 'single_subscribe_input_height' ) . 'px' );

	$css['global']['.single-subscribe .widget #wp-subscribe input.submit']['height'] = Books_Sanitize::size( books_get_settings( 'single_subscribe_input_height' ) . 'px' );

	$css['global']['.single-subscribe .widget #wp-subscribe input.submit']['line-height'] = Books_Sanitize::size( books_get_settings( 'single_subscribe_input_height' ) . 'px' );

	$css['global'][ books_implode( $subscribe_input_class ) ]['border-radius'] = Books_Sanitize::size( books_get_settings( 'single_subscribe_input_border_radius' ) . 'px' );

	// Subscribe Box Input border.
	books_merge_value( $css['global'][ books_implode( $subscribe_input_class ) ], Books_Sanitize::border2( books_get_settings( 'single_subscribe_input_border' ) ) );

	// Subscribe Box Submit button.
	$css['global']['.single-subscribe .widget #wp-subscribe input.submit']['background']    = Books_Sanitize::color( books_get_settings( 'single_subscribe_submit_backgroud' ) );
	$css['global']['.single-subscribe .widget #wp-subscribe input.submit']['border-radius'] = Books_Sanitize::size( books_get_settings( 'single_subscribe_submit_border_radius' ) . 'px' );

	// Subscribe Box Submit border.
	$subscribe_box_submit_border = Books_Sanitize::border( books_get_settings( 'single_subscribe_submit_border' ) );
	$css['global']['.single-subscribe .widget #wp-subscribe input.submit'][ $subscribe_box_submit_border ['direction'] ] = $subscribe_box_submit_border ['value'];

	books_merge_value( $css['global']['.single-subscribe .widget #wp-subscribe input.submit'], Books_Sanitize::padding( books_get_settings( 'single_subscribe_submit_padding' ) ) );

	// Author Box.
	$css['global']['.postauthor'] = Books_Sanitize::background( books_get_settings( 'authorbox_background' ) );

	books_merge_value( $css['global']['.postauthor'], Books_Sanitize::margin( books_get_settings( 'authorbox_margin' ) ) );

	books_merge_value( $css['global']['.postauthor'], Books_Sanitize::padding( books_get_settings( 'authorbox_padding' ) ) );

	$css['global']['.postauthor']['border-radius'] = Books_Sanitize::size( books_get_settings( 'authorbox_border_radius' ) . 'px' );

	// Box Shadow.
	if ( ! books_get_settings( 'authorbox_shadow' ) ) {
		$css['global']['.postauthor']['box-shadow'] = 'none';
	}

	// Border.
	books_merge_value( $css['global']['.postauthor'], Books_Sanitize::border2( books_get_settings( 'authorbox_border' ) ) );

	// Author image.
	books_merge_value( $css['global']['.postauthor img'], Books_Sanitize::margin( books_get_settings( 'single_author_image_margin' ) ) );

	$css['global']['.postauthor img']['border-radius'] = Books_Sanitize::size( books_get_settings( 'single_author_image_border_radius' ) . 'px' );

	// Single Page titles Styling.
	$titles_align = books_get_settings( 'single_title_alignment' );

	$titles_align_class = [
		'.comment-title',
		'#respond',
		'.related-posts-title',
	];
	$titles_class       = [
		'#respond h4',
		'.total-comments',
		'.related-posts h4',
	];

	$css['global'][ books_implode( $titles_class ) ] = Books_Sanitize::background( books_get_settings( 'single_title_background' ) );
	books_merge_value( $css['global'][ books_implode( $titles_class ) ], Books_Sanitize::padding( books_get_settings( 'single_title_padding' ) ) );
	// Single title border.
	$single_titles_border = Books_Sanitize::border( books_get_settings( 'single_title_border' ) );
	$css['global'][ books_implode( $titles_class ) ][ $single_titles_border ['direction'] ] = $single_titles_border ['value'];

	if ( 'left' === $titles_align ) :
		$css['global'][ books_implode( $titles_class ) ]['display'] = 'inline-block';
	elseif ( 'center' === $titles_align ) :
		$css['global'][ books_implode( $titles_align_class ) ]['text-align'] = 'center';
		$css['global'][ books_implode( $titles_class ) ]['display']          = 'inline-block';
	elseif ( 'right' === $titles_align ) :
		$css['global'][ books_implode( $titles_align_class ) ]['text-align'] = 'right';
		$css['global'][ books_implode( $titles_class ) ]['display']          = 'inline-block';
	endif;

	// Comments.
	// Margin and Padding.
	books_merge_value( $css['global']['#comments'], Books_Sanitize::margin( books_get_settings( 'comments_margin' ) ) );
	books_merge_value( $css['global']['#comments'], Books_Sanitize::padding( books_get_settings( 'comments_padding' ) ) );
	books_merge_value( $css['global']['#comments li'], Books_Sanitize::margin( books_get_settings( 'comments_list_margin' ) ) );
	books_merge_value( $css['global']['#comments li'], Books_Sanitize::padding( books_get_settings( 'comments_list_padding' ) ) );
	books_merge_value( $css['global']['#comments .avatar'], Books_Sanitize::margin( books_get_settings( 'comments_list_image_margin' ) ) );

	// Author Comment.
	books_merge_value( $css['global']['#comments li.bypostauthor'], Books_Sanitize::margin( books_get_settings( 'comments_list_author_margin' ) ) );
	books_merge_value( $css['global']['#comments li.bypostauthor'], Books_Sanitize::padding( books_get_settings( 'comments_list_author_padding' ) ) );
	$css['global']['#comments li.bypostauthor']['background-color'] = Books_Sanitize::color( books_get_settings( 'author_comment_bg' ) );

	// Comment list.
	// Border.
	$single_titles_border = Books_Sanitize::border( books_get_settings( 'comments_list_border' ) );
	$css['global']['#comments li'][ $single_titles_border ['direction'] ] = $single_titles_border ['value'];

	$css['global']['ol.commentlist']['border-top']        = $single_titles_border ['value'];
	$css['global']['#comments ul.children']['border-top'] = $single_titles_border ['value'];

	$css['global']['#comments .avatar']['border-radius'] = Books_Sanitize::size( books_get_settings( 'comments_list_image_border_radius' ) . 'px' );

	// Typography.
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'comments_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'comments_meta_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'comments_text_font' ) ) );

}

/**
 * Copyrights
 *
 * @param array $css Array of dynamic CSS.
 */
function books_copyrights( &$css ) {

	// copyrights border.
	$copyrights_border = Books_Sanitize::border( books_get_settings( 'copyrights_border' ) );
	$css['global']['.copyrights'][ $copyrights_border ['direction'] ] = $copyrights_border ['value'];

}

/**
 * Footer
 *
 * @param array $css Array of dynamic CSS.
 */
function books_footer( &$css ) {

	// Footer.
	if ( books_get_settings( 'mts_top_footer' ) ) {
		books_merge_value( $css['global']['#site-footer'], Books_Sanitize::margin( books_get_settings( 'mts_top_footer_margin' ) ) );
		books_merge_value( $css['global']['#site-footer'], Books_Sanitize::padding( books_get_settings( 'mts_top_footer_padding' ) ) );
	}

	// Copyrights.
	books_merge_value( $css['global']['.copyrights'], Books_Sanitize::margin( books_get_settings( 'copyrights_margin' ) ) );
	books_merge_value( $css['global']['.copyrights'], Books_Sanitize::padding( books_get_settings( 'copyrights_padding' ) ) );

}

/**
 * Archive
 *
 * @param array $css Array of dynamic CSS.
 */
function books_archive_post( &$css ) {

	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'archive_post_title_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'archive_post_excerpt_font' ) ) );
	books_map_css_selectors( $css['global'], Books_Sanitize::typography( books_get_settings( 'archive_meta_info_font' ) ) );

}

/**
 * Misc
 *
 * @param array $css Array of dynamic CSS.
 */
function books_misc_css( &$css ) {

	// Show Logo.
	if ( ! books_get_settings( 'hide_logo' ) ) {
		$css['global']['.logo-wrap']['display'] = 'none';
	}

	// Back to top.
	$css['global']['#move-to-top'] = Books_Sanitize::background( books_get_settings( 'top_button_background' ) );

	$css['global']['#move-to-top:hover'] = Books_Sanitize::background( books_get_settings( 'top_button_background_hover' ) );

	// Border.
	$top_button_border = Books_Sanitize::border( books_get_settings( 'top_button_border' ) );
	$css['global']['#move-to-top'][ $top_button_border ['direction'] ] = $top_button_border ['value'];

	// Font-size, Padding and Position.
	$css['global']['#move-to-top .fa']['font-size'] = Books_Sanitize::size( books_get_settings( 'top_button_font_size' ) . 'px' );
	books_merge_value( $css['global']['#move-to-top'], Books_Sanitize::padding( books_get_settings( 'top_button_padding' ) ) );

	$top_button_position = books_get_settings( 'top_button_position' );
	if ( $top_button_position && is_array( $top_button_position ) ) {
		foreach ( $top_button_position as $key => $position ) {
			$css['global']['#move-to-top'][ $key ] = $position;
		}
	}

	// Border-radius.
	$css['global']['#move-to-top']['border-radius'] = Books_Sanitize::size( books_get_settings( 'top_button_border_radius' ) . 'px' );

	// Colors.
	$css['global']['#move-to-top']['color']       = Books_Sanitize::color( books_get_settings( 'top_button_color' ) );
	$css['global']['#move-to-top:hover']['color'] = Books_Sanitize::color( books_get_settings( 'top_button_color_hover' ) );

	$css['global']['#move-to-top:hover']['border-color'] = Books_Sanitize::color( books_get_settings( 'top_button_background_hover' ) );

	// Full width Related Posts.
	if ( 'full' === books_get_settings( 'related_posts_position' ) ) {
		$css['global']['.single .article']['margin-bottom'] = '0px';
	}

}

/**
 * WooCommerce
 *
 * @param array $css Array of dynamic CSS.
 */
function books_woo( &$css ) {

	$woo_classes        = [
		'.woocommerce nav.woocommerce-pagination ul li a',
		'.woocommerce-page nav.woocommerce-pagination ul li a',
	];
	$woo_hover_classes  = [
		'.woocommerce nav.woocommerce-pagination ul li a:hover',
		'.woocommerce nav.woocommerce-pagination ul li a:focus',
		'.woocommerce-page nav.woocommerce-pagination ul li a:hover',
		'.woocommerce-page nav.woocommerce-pagination ul li a:focus',
	];
	$woo_active_classes = [
		'.woocommerce nav.woocommerce-pagination ul li span.current',
		'.woocommerce-page nav.woocommerce-pagination ul li span.current',
	];

	$css['global'][ books_implode( $woo_classes ) ] = Books_Sanitize::background( books_get_settings( 'mts_pagenavigation_bgcolor' ) );

	$css['global'][ books_implode( $woo_hover_classes ) ] = Books_Sanitize::background( books_get_settings( 'mts_pagenavigation_hover_bgcolor' ) );

	$css['global'][ books_implode( $woo_active_classes ) ] = Books_Sanitize::background( books_get_settings( 'mts_pagenavigation_active_bgcolor' ) );

}
