<?php
/**
 * Additional helper functions that the framework or themes may use.
 * The functions in this file are functions that don't really have a home within any other parts of the framework.
 *
 * @package Books
 */

use MyThemeShop\Helpers\Conditional;
use MyThemeShop\Helpers\Str;
use MyThemeShop\Helpers\WordPress;

/**
 * Debug function.
 */
if ( ! function_exists( 'dd' ) ) {
	function dd() {
		array_map( function ( $x ) {
			print_r( $x );
			echo '<br>';
		}, func_get_args() );
		die( 1 );
	}
}

if ( ! function_exists( 'books_get_cat_name' ) ) {
	function books_get_cat_name( $slug ) {
		$category = get_term_by( 'slug', $slug, 'category' );
		if ( ! empty( $category ) ) {
			return $category->name;
		} else {
			return '';
		}
	}
}

/**
 * Check if the string begins with the given value
 *
 * @param  string $needle   The sub-string to search for.
 * @param  string $haystack The string to search.
 *
 * @return bool
 */
function books_str_start_with( $needle, $haystack ) {
	return Str::starts_with( $needle, $haystack );
}

/**
 * Check if the string contains the given value
 *
 * @param  string $needle   The sub-string to search for.
 * @param  string $haystack The string to search.
 *
 * @return bool
 */
function books_str_contains( $needle, $haystack ) {
	return Str::contains( $needle, $haystack );
}

/**
 * Function for setting the content width of a theme.
 *
 * @param  int $width for comtent width.
 * @return void
 */
function books_set_content_width( $width = '' ) {
	$GLOBALS['content_width'] = absint( $width );
}

/**
 * Get settings from theme options
 *
 * @param  string $id setting id.
 * @param  mixed  $default setting default value.
 * @return mixed
 */
function books_get_settings( $id, $default = false ) {
	return books()->settings->get( $id, $default );
}

/**
 * This function transforms the php.ini notation for numbers (like '2M') to an integer.
 *
 * @static
 * @access public
 * @since 3.8.0
 * @param string $size The size.
 * @return int
 */
function books_let_to_num( $size ) {
	return Str::let_to_num( $size );
}

/**
 * Instantiates the WordPress filesystem for use with books.
 *
 * @return object
 */
function books_init_filesystem() {
	return WordPress::get_filesystem();
}

/**
 * Increase excerpt length to 100.
 *
 * @param int $length
 *
 * @return int
 */
function books_excerpt_length( $length ) {
	return 100;
}
add_filter( 'excerpt_length', 'books_excerpt_length', 20 );

/**
 * Remove [...] and shortcodes from excerpts
 *
 * @param string $output
 *
 * @return string
 */
function books_custom_excerpt( $output ) {
	return preg_replace( '/\[[^\]]*]/', '', $output );
}
add_filter( 'get_the_excerpt', 'books_custom_excerpt' );

/**
 * Get HTML-escaped excerpt up to the specified length.
 *
 * @param int $limit
 *
 * @return string
 */
if ( ! function_exists( 'books_excerpt' ) ) {
	function books_excerpt( $limit = 55 ) {
		return esc_html( books_truncate( get_the_excerpt(), $limit, 'words' ) );
	}
}

/**
 * Truncate string to x letters/words.
 *
 * @param string $str
 * @param int $length
 * @param string $units
 * @param string $ellipsis
 *
 * @return string
 */
if ( ! function_exists( 'books_truncate' ) ) {
	function books_truncate( $str, $length = 40, $units = 'letters', $ellipsis = '&nbsp;&hellip;' ) {
		if ( 'letters' === $units ) {
			if ( mb_strlen( $str ) > $length ) {
				return mb_substr( $str, 0, $length ) . $ellipsis;
			} else {
				return $str;
			}
		} else {
			return wp_trim_words( $str, $length, $ellipsis );
		}
	}
}

/**
 * Check if excerpt is empty and
 * Get HTML-escaped excerpt up to the specified length.
 *
 * @param int $limit
 *
 * @return string
 */
if ( ! function_exists( 'get_books_excerpt' ) ) {
	function get_books_excerpt( $limit = 55 ) {
		if ( empty( books_excerpt() ) ) {
			return books_truncate( get_the_content(), $limit, 'words' );
		} else {
			return books_excerpt( $limit );
		}
	}
}

/**
 * Shorthand function to check for more tag in post.
 *
 * @return bool|int
 */
if ( ! function_exists( 'books_post_has_moretag' ) ) {
	function books_post_has_moretag() {
		return strpos( get_the_content(), '<!--more-->' );
	}
}

/**
 * Change the HTML markup of the post thumbnail.
 *
 * @param string $html    html structure.
 * @param int    $post_id    post ID.
 * @param string $post_image_id    featured image ID.
 * @param int    $size    image size.
 * @param string $attr    image attributes.
 *
 * @return string
 */
function books_post_image_html( $html, $post_id, $post_image_id, $size, $attr ) {

	// use featured image.
	if ( has_post_thumbnail( $post_id ) || 'shop_thumbnail' === $size ) {
		return $html;
	}

	// use first attached image.
	$images = get_children( 'post_type=attachment&post_mime_type=image&suppress_filters=false&post_parent=' . $post_id );
	if ( ! empty( $images ) ) {
		$image = reset( $images );
		return wp_get_attachment_image( $image->ID, $size, false, $attr );
	}

	// use no preview fallback.
	if ( file_exists( get_template_directory() . '/images/nothumb-' . $size . '.png' ) ) {
		$placeholder = get_template_directory_uri() . '/images/nothumb-' . $size . '.png';

		$placeholder_src = $placeholder;
		$layzr_attr      = '';

		$placeholder_classs = 'attachment-' . $size . ' wp-post-image';
		return '<img src="' . esc_url( $placeholder_src ) . '" class="' . esc_attr( $placeholder_classs ) . '" alt="' . esc_attr( get_the_title() ) . '"' . $layzr_attr . '>';
	}
	return '';
}
add_filter( 'post_thumbnail_html', 'books_post_image_html', 10, 5 );

// NAV ------------------------------------------------------------------!
if ( ! function_exists( 'books_get_menu_location_name' ) ) {
	/**
	 * Function for grabbing a WP nav menu theme location name.
	 *
	 * @param string $location menu location name.
	 *
	 * @return string
	 */
	function books_get_menu_location_name( $location ) {
		$locations = get_registered_nav_menus();
		return isset( $locations[ $location ] ) ? $locations[ $location ] : '';
	}
}

if ( ! function_exists( 'books_get_menu_name' ) ) {
	/**
	 * Function for grabbing a WP nav menu name based on theme location.
	 *
	 * @param string $location menu name.
	 *
	 * @return string
	 */
	function books_get_menu_name( $location ) {
		$locations = get_nav_menu_locations();
		return isset( $locations[ $location ] ) ? wp_get_nav_menu_object( $locations[ $location ] )->name : '';
	}
}

// SIDEBAR ------------------------------------------------------------------!
/**
 * Function for grabbing a dynamic sidebar name.
 *
 * @global array   $wp_registered_sidebars
 *
 * @param string $sidebar_id sidebar ID.
 *
 * @return string
 */
function books_get_sidebar_name( $sidebar_id ) {
	global $wp_registered_sidebars;
	return isset( $wp_registered_sidebars[ $sidebar_id ] ) ? $wp_registered_sidebars[ $sidebar_id ]['name'] : '';
}

// Conditional --------------------------------------------------------------!
/**
 * Check if WooCommerce is active or not
 *
 * @return boolean
 */
function books_is_woocommerce_active() {
	if ( Conditional::is_woocommerce_active() ) {
		return true;
	}
}

/**
 * Check if WooCommerce page
 *
 * @return boolean
 */
function is_woocommerce_page() {
	if ( function_exists( 'is_woocommerce' ) && is_woocommerce() ) {
		return true;
	}
}

/**
 * Check if MyComposer is active or not
 *
 * @return boolean
 */
function books_is_composer() {
	return class_exists( 'MyComposer' );
}

/**
 * Check if ipad
 *
 * @return boolean
 */
function books_is_ipad() {
	$is_ipad = (bool) ( isset( $_SERVER['HTTP_USER_AGENT'] ) && false !== strpos( $_SERVER['HTTP_USER_AGENT'], 'iPad' ) );

	return $is_ipad;
}

/**
 * Check if elementor enable
 *
 * @param string $location check page location.
 *
 * @return boolean
 */
function books_is_elementor( $location ) {
	if ( function_exists( 'elementor_theme_do_location' ) && elementor_theme_do_location( $location ) ) {
		return true;
	}
}

/**
 * Return a px based font-size to rem.
 *
 * @param string $size font-size needed to convert.
 *
 * @return string converted.
 */
function books_px2rem( $size ) {
	$size = (int) $size;
	return ( $size / 16 ) . 'rem';
}

/**
 * Return left and right padding for full width image.
 *
 * @param array $paddings padding top|right|bottom|left.
 *
 * @return int calculated padding left+right.
 */
function books_get_total_padding( $paddings ) {

	if ( ! $paddings && ! is_array( $paddings ) ) {
		return;
	}

	$new_padding = (float) $paddings['padding-left'] + (float) $paddings['padding-right'];
	$check_unit  = books_str_contains( 'rem', $paddings['padding-left'] . $paddings['padding-right'] ) ? 'rem' : 'px';

	$total = $new_padding;
	if ( 'rem' === $check_unit ) {
		$total = $new_padding * 16;
	}

	return $total;
}

// Data ----------------------------------------------------------------------.

/**
 * Get post id for slider.
 *
 * @return boolean|int
 */
function books_get_slider_page_id() {

	if ( is_search() ) {
		return false;
	}

	$object_id = get_queried_object_id();

	if ( ! is_home() && ! is_front_page() && ! is_archive() && isset( $object_id ) ) {
		return $object_id;
	}

	if ( ! is_home() && is_front_page() && isset( $object_id ) ) {
		return $object_id;
	}

	if ( is_home() && ! is_front_page() ) {
		return get_option( 'page_for_posts' );
	}

	if ( class_exists( 'WooCommerce' ) && is_shop() ) {
		return get_option( 'woocommerce_shop_page_id' );
	}

	return false;
}

/**
 * MTS icons for use in nav menus and icon select option.
 *
 * @return array
 */
function books_get_icons() {
	// @codingStandardsIgnoreStart
	$icons = array(
		__( 'Web Application Icons', 'books' ) => array(
			'address-book', 'address-book-o', 'address-card', 'address-card-o', 'adjust', 'american-sign-language-interpreting', 'anchor', 'archive', 'area-chart', 'arrows', 'arrows-h', 'arrows-v', 'asl-interpreting', 'assistive-listening-systems', 'asterisk', 'at', 'audio-description', 'automobile', 'balance-scale', 'ban', 'bank', 'bar-chart', 'bar-chart-o', 'barcode', 'bars', 'bath', 'bathtub', 'battery', 'battery-0', 'battery-1', 'battery-2', 'battery-3', 'battery-4', 'battery-empty', 'battery-full', 'battery-half', 'battery-quarter', 'battery-three-quarters', 'bed', 'beer', 'bell', 'bell-o', 'bell-slash', 'bell-slash-o', 'bicycle', 'binoculars', 'birthday-cake', 'blind', 'bluetooth', 'bluetooth-b', 'bolt', 'bomb', 'book', 'bookmark', 'bookmark-o', 'braille', 'briefcase', 'bug', 'building', 'building-o', 'bullhorn', 'bullseye', 'bus', 'cab', 'calculator', 'calendar', 'calendar-check-o', 'calendar-minus-o', 'calendar-o', 'calendar-plus-o', 'calendar-times-o', 'camera', 'camera-retro', 'car', 'caret-square-o-down', 'caret-square-o-left', 'caret-square-o-right', 'caret-square-o-up', 'cart-arrow-down', 'cart-plus', 'cc', 'certificate', 'check', 'check-circle', 'check-circle-o', 'check-square', 'check-square-o', 'child', 'circle', 'circle-o', 'circle-o-notch', 'circle-thin', 'clock-o', 'clone', 'close', 'cloud', 'cloud-download', 'cloud-upload', 'code', 'code-fork', 'coffee', 'cog', 'cogs', 'comment', 'comment-o', 'commenting', 'commenting-o', 'comments', 'comments-o', 'compass', 'copyright', 'creative-commons', 'credit-card', 'credit-card-alt', 'crop', 'crosshairs', 'cube', 'cubes', 'cutlery', 'dashboard', 'database', 'deaf', 'deafness', 'desktop', 'diamond', 'dot-circle-o', 'download', 'drivers-license', 'drivers-license-o', 'edit', 'ellipsis-h', 'ellipsis-v', 'envelope', 'envelope-o', 'envelope-open', 'envelope-open-o', 'envelope-square', 'eraser', 'exchange', 'exclamation', 'exclamation-circle', 'exclamation-triangle', 'external-link', 'external-link-square', 'eye', 'eye-slash', 'eyedropper', 'fax', 'feed', 'female', 'fighter-jet', 'file-archive-o', 'file-audio-o', 'file-code-o', 'file-excel-o', 'file-image-o', 'file-movie-o', 'file-pdf-o', 'file-photo-o', 'file-picture-o', 'file-powerpoint-o', 'file-sound-o', 'file-video-o', 'file-word-o', 'file-zip-o', 'film', 'filter', 'fire', 'fire-extinguisher', 'flag', 'flag-checkered', 'flag-o', 'flash', 'flask', 'folder', 'folder-o', 'folder-open', 'folder-open-o', 'frown-o', 'futbol-o', 'gamepad', 'gavel', 'gear', 'gears', 'gift', 'glass', 'globe', 'graduation-cap', 'group', 'hand-grab-o', 'hand-lizard-o', 'hand-paper-o', 'hand-peace-o', 'hand-pointer-o', 'hand-rock-o', 'hand-scissors-o', 'hand-spock-o', 'hand-stop-o', 'handshake-o', 'hard-of-hearing', 'hashtag', 'hdd-o', 'headphones', 'heart', 'heart-o', 'heartbeat', 'history', 'home', 'hotel', 'hourglass', 'hourglass-1', 'hourglass-2', 'hourglass-3', 'hourglass-end', 'hourglass-half', 'hourglass-o', 'hourglass-start', 'i-cursor', 'id-badge', 'id-card', 'id-card-o', 'image', 'inbox', 'industry', 'info', 'info-circle', 'institution', 'key', 'keyboard-o', 'language', 'laptop', 'leaf', 'legal', 'lemon-o', 'level-down', 'level-up', 'life-bouy', 'life-buoy', 'life-ring', 'life-saver', 'lightbulb-o', 'line-chart', 'location-arrow', 'lock', 'low-vision', 'magic', 'magnet', 'mail-forward', 'mail-reply', 'mail-reply-all', 'male', 'map', 'map-marker', 'map-o', 'map-pin', 'map-signs', 'meh-o', 'microchip', 'microphone', 'microphone-slash', 'minus', 'minus-circle', 'minus-square', 'minus-square-o', 'mobile', 'mobile-phone', 'money', 'moon-o', 'mortar-board', 'motorcycle', 'mouse-pointer', 'music', 'navicon', 'newspaper-o', 'object-group', 'object-ungroup', 'paint-brush', 'paper-plane', 'paper-plane-o', 'paw', 'pencil', 'pencil-square', 'pencil-square-o', 'percent', 'phone', 'phone-square', 'photo', 'picture-o', 'pie-chart', 'plane', 'plug', 'plus', 'plus-circle', 'plus-square', 'plus-square-o', 'podcast', 'power-off', 'print', 'puzzle-piece', 'qrcode', 'question', 'question-circle', 'question-circle-o', 'quote-left', 'quote-right', 'random', 'recycle', 'rebooks', 'registered', 'remove', 'reorder', 'reply', 'reply-all', 'retweet', 'road', 'rocket', 'rss', 'rss-square', 's15', 'search', 'search-minus', 'search-plus', 'send', 'send-o', 'server', 'share', 'share-alt', 'share-alt-square', 'share-square', 'share-square-o', 'shield', 'ship', 'shopping-bag', 'shopping-basket', 'shopping-cart', 'shower', 'sign-in', 'sign-language', 'sign-out', 'signal', 'signing', 'sitemap', 'sliders', 'smile-o', 'snowflake-o', 'soccer-ball-o', 'sort', 'sort-alpha-asc', 'sort-alpha-desc', 'sort-amount-asc', 'sort-amount-desc', 'sort-asc', 'sort-desc', 'sort-down', 'sort-numeric-asc', 'sort-numeric-desc', 'sort-up', 'space-shuttle', 'spinner', 'spoon', 'square', 'square-o', 'star', 'star-half', 'star-half-empty', 'star-half-full', 'star-half-o', 'star-o', 'sticky-note', 'sticky-note-o', 'street-view', 'suitcase', 'sun-o', 'support', 'tablet', 'tachometer', 'tag', 'tags', 'tasks', 'taxi', 'television', 'terminal', 'thermometer', 'thermometer-0', 'thermometer-1', 'thermometer-2', 'thermometer-3', 'thermometer-4', 'thermometer-empty', 'thermometer-full', 'thermometer-half', 'thermometer-quarter', 'thermometer-three-quarters', 'thumb-tack', 'thumbs-down', 'thumbs-o-down', 'thumbs-o-up', 'thumbs-up', 'ticket', 'times', 'times-circle', 'times-circle-o', 'times-rectangle', 'times-rectangle-o', 'tint', 'toggle-down', 'toggle-left', 'toggle-off', 'toggle-on', 'toggle-right', 'toggle-up', 'trademark', 'trash', 'trash-o', 'tree', 'trophy', 'truck', 'tty', 'tv', 'umbrella', 'universal-access', 'university', 'unlock', 'unlock-alt', 'unsorted', 'upload', 'user', 'user-circle', 'user-circle-o', 'user-o', 'user-plus', 'user-secret', 'user-times', 'users', 'vcard', 'vcard-o', 'video-camera', 'volume-control-phone', 'volume-down', 'volume-off', 'volume-up', 'warning', 'wheelchair', 'wheelchair-alt', 'wifi', 'window-close', 'window-close-o', 'window-maximize', 'window-minimize', 'window-restore', 'wrench'
		),
		__( 'Accessibility Icons', 'books' ) => array(
			'american-sign-language-interpreting', 'asl-interpreting', 'assistive-listening-systems', 'audio-description', 'blind', 'braille', 'cc', 'deaf', 'deafness', 'hard-of-hearing', 'low-vision', 'question-circle-o', 'sign-language', 'signing', 'tty', 'universal-access', 'volume-control-phone', 'wheelchair', 'wheelchair-alt'
		),
		__( 'Hand Icons', 'books' ) => array(
			'hand-grab-o', 'hand-lizard-o', 'hand-o-down', 'hand-o-left', 'hand-o-right', 'hand-o-up', 'hand-paper-o', 'hand-peace-o', 'hand-pointer-o', 'hand-rock-o', 'hand-scissors-o', 'hand-spock-o', 'hand-stop-o', 'thumbs-down', 'thumbs-o-down', 'thumbs-o-up', 'thumbs-up'
		),
		__( 'Transportation Icons', 'books' ) => array(
			'ambulance', 'automobile', 'bicycle', 'bus', 'cab', 'car', 'fighter-jet', 'motorcycle', 'plane', 'rocket', 'ship', 'space-shuttle', 'subway', 'taxi', 'train', 'truck', 'wheelchair', 'wheelchair-alt'
		),
		__( 'Gender Icons', 'books' ) => array(
			'genderless', 'intersex', 'mars', 'mars-double', 'mars-stroke', 'mars-stroke-h', 'mars-stroke-v', 'mercury', 'neuter', 'transgender', 'transgender-alt', 'venus', 'venus-double', 'venus-mars'
		),
		__( 'File Type Icons', 'books' ) => array(
			'file', 'file-archive-o', 'file-audio-o', 'file-code-o', 'file-excel-o', 'file-image-o', 'file-movie-o', 'file-o', 'file-pdf-o', 'file-photo-o', 'file-picture-o', 'file-powerpoint-o', 'file-sound-o', 'file-text', 'file-text-o', 'file-video-o', 'file-word-o', 'file-zip-o'
		),
		__( 'Spinner Icons', 'books' ) => array(
			'circle-o-notch', 'cog', 'gear', 'rebooks', 'spinner'
		),
		__( 'Form Control Icons', 'books' ) => array(
			'check-square', 'check-square-o', 'circle', 'circle-o', 'dot-circle-o', 'minus-square', 'minus-square-o', 'plus-square', 'plus-square-o', 'square', 'square-o'
		),
		__( 'Payment Icons', 'books' ) => array(
			'cc-amex', 'cc-diners-club', 'cc-discover', 'cc-jcb', 'cc-mastercard', 'cc-paypal', 'cc-stripe', 'cc-visa', 'credit-card', 'credit-card-alt', 'google-wallet', 'paypal'
		),
		__( 'Chart Icons', 'books' ) => array(
			'area-chart', 'bar-chart', 'bar-chart-o', 'line-chart', 'pie-chart'
		),
		__( 'Currency Icons', 'books' ) => array(
			'bitcoin', 'btc', 'cny', 'dollar', 'eur', 'euro', 'gbp', 'gg', 'gg-circle', 'ils', 'inr', 'jpy', 'krw', 'money', 'rmb', 'rouble', 'rub', 'ruble', 'rupee', 'shekel', 'sheqel', 'try', 'turkish-lira', 'usd', 'won', 'yen'
		),
		__( 'Text Editor Icons', 'books' ) => array(
			'align-center', 'align-justify', 'align-left', 'align-right', 'bold', 'chain', 'chain-broken', 'clipboard', 'columns', 'copy', 'cut', 'dedent', 'eraser', 'file', 'file-o', 'file-text', 'file-text-o', 'files-o', 'floppy-o', 'font', 'header', 'indent', 'italic', 'link', 'list', 'list-alt', 'list-ol', 'list-ul', 'outdent', 'paperclip', 'paragraph', 'paste', 'repeat', 'rotate-left', 'rotate-right', 'save', 'scissors', 'strikethrough', 'subscript', 'superscript', 'table', 'text-height', 'text-width', 'th', 'th-large', 'th-list', 'underline', 'undo', 'unlink'
		),
		__( 'Directional Icons', 'books' ) => array(
			'angle-double-down', 'angle-double-left', 'angle-double-right', 'angle-double-up', 'angle-down', 'angle-left', 'angle-right', 'angle-up', 'arrow-circle-down', 'arrow-circle-left', 'arrow-circle-o-down', 'arrow-circle-o-left', 'arrow-circle-o-right', 'arrow-circle-o-up', 'arrow-circle-right', 'arrow-circle-up', 'arrow-down', 'arrow-left', 'arrow-right', 'arrow-up', 'arrows', 'arrows-alt', 'arrows-h', 'arrows-v', 'caret-down', 'caret-left', 'caret-right', 'caret-square-o-down', 'caret-square-o-left', 'caret-square-o-right', 'caret-square-o-up', 'caret-up', 'chevron-circle-down', 'chevron-circle-left', 'chevron-circle-right', 'chevron-circle-up', 'chevron-down', 'chevron-left', 'chevron-right', 'chevron-up', 'exchange', 'hand-o-down', 'hand-o-left', 'hand-o-right', 'hand-o-up', 'long-arrow-down', 'long-arrow-left', 'long-arrow-right', 'long-arrow-up', 'toggle-down', 'toggle-left', 'toggle-right', 'toggle-up'
		),
		__( 'Video Player Icons', 'books' ) => array(
			'arrows-alt', 'backward', 'compress', 'eject', 'expand', 'fast-backward', 'fast-forward', 'forward', 'pause', 'pause-circle', 'pause-circle-o', 'play', 'play-circle', 'play-circle-o', 'random', 'step-backward', 'step-forward', 'stop', 'stop-circle', 'stop-circle-o', 'youtube-play'
		),
		__( 'Brand Icons', 'books' ) => array(
			'500px', 'adn', 'amazon', 'android', 'angellist', 'apple', 'bandcamp', 'behance', 'behance-square', 'bitbucket', 'bitbucket-square', 'bitcoin', 'black-tie', 'bluetooth', 'bluetooth-b', 'btc', 'buysellads', 'cc-amex', 'cc-diners-club', 'cc-discover', 'cc-jcb', 'cc-mastercard', 'cc-paypal', 'cc-stripe', 'cc-visa', 'chrome', 'codepen', 'codiepie', 'connectdevelop', 'contao', 'css3', 'dashcube', 'delicious', 'deviantart', 'digg', 'dribbble', 'dropbox', 'drupal', 'edge', 'eercast', 'empire', 'envira', 'etsy', 'expeditedssl', 'fa', 'facebook', 'facebook-f', 'facebook-official', 'facebook-square', 'firefox', 'first-order', 'flickr', 'font-awesome', 'fonticons', 'fort-awesome', 'forumbee', 'foursquare', 'free-code-camp', 'ge', 'get-pocket', 'gg', 'gg-circle', 'git', 'git-square', 'github', 'github-alt', 'github-square', 'gitlab', 'gittip', 'glide', 'glide-g', 'google', 'google-plus', 'google-plus-circle', 'google-plus-official', 'google-plus-square', 'google-wallet', 'gratipay', 'grav', 'hacker-news', 'houzz', 'html5', 'imdb', 'instagram', 'internet-explorer', 'ioxhost', 'joomla', 'jsfiddle', 'lastfm', 'lastfm-square', 'leanpub', 'linkedin', 'linkedin-square', 'linode', 'linux', 'maxcdn', 'meanpath', 'medium', 'meetup', 'mixcloud', 'modx', 'odnoklassniki', 'odnoklassniki-square', 'opencart', 'openid', 'opera', 'optin-monster', 'pagelines', 'paypal', 'pied-piper', 'pied-piper-alt', 'pied-piper-pp', 'pinterest', 'pinterest-p', 'pinterest-square', 'product-hunt', 'qq', 'quora', 'ra', 'ravelry', 'rebel', 'reddit', 'reddit-alien', 'reddit-square', 'renren', 'resistance', 'safari', 'scribd', 'sellsy', 'share-alt', 'share-alt-square', 'shirtsinbulk', 'simplybuilt', 'skyatlas', 'skype', 'slack', 'slideshare', 'snapchat', 'snapchat-ghost', 'snapchat-square', 'soundcloud', 'spotify', 'stack-exchange', 'stack-overflow', 'steam', 'steam-square', 'stumbleupon', 'stumbleupon-circle', 'superpowers', 'telegram', 'tencent-weibo', 'themeisle', 'trello', 'tripadvisor', 'tumblr', 'tumblr-square', 'twitch', 'twitter', 'twitter-square', 'usb', 'viacoin', 'viadeo', 'viadeo-square', 'vimeo', 'vimeo-square', 'vine', 'vk', 'wechat', 'weibo', 'weixin', 'whatsapp', 'wikipedia-w', 'windows', 'wordpress', 'wpbeginner', 'wpexplorer', 'wpforms', 'xing', 'xing-square', 'y-combinator', 'y-combinator-square', 'yahoo', 'yc', 'yc-square', 'yelp', 'yoast', 'youtube', 'youtube-play', 'youtube-square'
		),
		__( 'Medical Icons', 'books' ) => array(
			'ambulance', 'h-square', 'heart', 'heart-o', 'heartbeat', 'hospital-o', 'medkit', 'plus-square', 'stethoscope', 'user-md', 'wheelchair', 'wheelchair-alt'
		)
	);
	// @codingStandardsIgnoreEnd

	return $icons;
}

/**
 * Wrap videos in .responsive-video div
 *
 * @param $html
 * @param $url
 * @param $attr
 *
 * @return string
 */
function books_responsive_video( $html, $url, $attr ) {

	// Only video embeds.
	$video_providers = [
		'youtube',
		'vimeo',
		'booksmotion',
		'wordpress.tv',
		'vine.co',
		'animoto',
		'blip.tv',
		'collegehumor.com',
		'funnyordie.com',
		'hulu.com',
		'revision3.com',
		'ted.com',
	];

	// Allow user to wrap other embeds.
	$providers = apply_filters( 'books_responsive_video', $video_providers );
	foreach ( $providers as $provider ) {
		if ( strstr( $url, $provider ) ) {
			$html = '<div class="flex-video flex-video-' . sanitize_html_class( $provider ) . '">' . $html . '</div>';
			break; // Break if video found.
		}
	}

	return $html;
}
add_filter( 'embed_oembed_html', 'books_responsive_video', 99, 3 );

// Comments ------------------------------------------------------------------!
/**
 * Exclude trackbacks from the comment count.
 *
 * @param $count
 *
 * @return int
 */
function books_comment_count( $count ) {
	if ( ! is_admin() ) {
		global $id;
		$comments         = get_comments( 'status=approve&post_id=' . $id );
		$comments_by_type = separate_comments( $comments );
		return count( $comments_by_type['comment'] );
	} else {
		return $count;
	}
}
add_filter( 'get_comments_number', 'books_comment_count', 0 );

/**
 * Add `has_thumb` to the post's class name if it has a thumbnail.
 *
 * @param $classes
 *
 * @return array
 */
function books_post_has_thumb_class( $classes ) {
	if ( has_post_thumbnail( get_the_ID() ) ) {
		$classes[] = 'has_thumb';
	}
		return $classes;
}
add_filter( 'post_class', 'books_post_has_thumb_class' );

/**
 * Custom `<article>` class name.
 */
if ( ! function_exists( 'books_article_class' ) ) {
	function books_get_article_class() {
		$classes[] = 'article';

		// sidebar or full width.
		if ( 'mts_nosidebar' === mts_custom_sidebar() ) {
			$classes[] = 'ss-full-width';
		}

		return $classes;
	}

	function books_article_class() {
		$classes = books_get_article_class();
		echo esc_html( join( ' ', $classes ) );
	}
}

if ( ! function_exists( 'books_single_page_class' ) ) {
	/**
	 * Custom `#page` class name.
	 */
	function books_single_page_class() {
		$class = '';

		if ( is_single() || is_page() || is_woocommerce_page() ) {
			$class = 'single clearfix';

			if ( is_single() ) {
				$class .= ' single_post_wrap';
			}

			if ( is_page() ) {
				$class .= ' single_page';
			}

			if ( is_woocommerce_page() ) {
				$class .= ' wc_page clearfix';
			}

			$header_animation = books_get_post_header_effect();
			if ( ! empty( $header_animation ) ) {
				$class .= ' ' . $header_animation;
			}
		}

		echo esc_attr( $class );
	}
}

/**
 * Single Post Pagination - Numbers + Previous/Next.
 * @param $args
 * @return mixed
 */
function books_wp_link_pages_args( $args ) {
	global $page, $numpages, $more, $pagenow;
	if ( 'next_and_number' != $args['next_or_number'] ) {
		return $args;
	}

	$args['next_or_number'] = 'number';

	if ( ! $more ) {
		return $args;
	}

	if ( $page - 1 ) {
		$args['before'] .= _wp_link_page( $page - 1 ) . $args['link_before'] . $args['previouspagelink'] . $args['link_after'] . '</a>';
	}

	if ( $page < $numpages ) {
		$args['after'] = _wp_link_page( $page + 1 ) . $args['link_before'] . $args['nextpagelink'] . $args['link_after'] . '</a>' . $args['after'];
	}

	return $args;
}
add_filter( 'wp_link_pages_args', 'books_wp_link_pages_args' );

if ( ! function_exists( 'books_get_post_header_effect' ) ) {
	/**
	 * Get Post header animation.
	 *
	 * @return string
	 */
	function books_get_post_header_effect() {
		$postheader_effect = get_post_meta( get_the_ID(), '_mts_postheader', true );

		return $postheader_effect;
	}
}

/**
 * Add Custom Gravatar Support.
 *
 * @param string $avatar_defaults avatar URL.
 *
 * @return mixed
 */
function books_custom_gravatar( $avatar_defaults ) {
	$books_avatar                     = get_template_directory_uri() . '/images/gravatar.png';
	$avatar_defaults[ $books_avatar ] = __( 'Custom Gravatar ( /images/gravatar.png )', 'books' );
	return $avatar_defaults;
}
add_filter( 'avatar_defaults', 'books_custom_gravatar' );

/**
 * Add `#main-menu` the WP Mega Menu's
 *
 * @param string $selector    container class.
 * @return string
 */
function books_megamenu_parent_element( $selector ) {
	return '.navigation';
}
add_filter( 'wpmm_container_selector', 'books_megamenu_parent_element' );

/**
 * Change the image size of WP Mega Menu's thumbnails.
 *
 * @param string $thumbnail_html thumbnail.
 * @param int    $post_id post ID.
 *
 * @return string
 */
function books_megamenu_thumbnails( $thumbnail_html, $post_id ) {
	$thumbnail_html  = '<div class="wpmm-thumbnail">';
	$thumbnail_html .= '<a title="' . get_the_title( $post_id ) . '" href="' . get_permalink( $post_id ) . '">';
	if ( has_post_thumbnail( $post_id ) ) :
		$thumbnail_html .= get_the_post_thumbnail( $post_id, 'books-widgetfull', array( 'title' => '' ) );
	else :
		$thumbnail_html .= '<img src="' . get_template_directory_uri() . '/images/nothumb-books-widgetfull.png" alt="' . __( 'No Preview', 'books' ) . '"  class="wp-post-image" />';
	endif;
	$thumbnail_html .= '</a>';

	// WP Review.
	$thumbnail_html .= ( function_exists( 'wp_review_show_total' ) ? wp_review_show_total( false ) : '' );
	$thumbnail_html .= '</div>';

	return $thumbnail_html;
}
add_filter( 'wpmm_thumbnail_html', 'books_megamenu_thumbnails', 10, 2 );

/**
 * Generate html attribute string for array.
 *
 * @param  array  $attributes Contains key/value pair to generate a string.
 * @param  string $prefix     If you want to append a prefic before every key.
 * @return string
 */
function books_generate_attributes( $attributes = [], $prefix = '' ) {

	// If empty return false.
	if ( empty( $attributes ) ) {
		return false;
	}

	$out = '';
	foreach ( $attributes as $key => $value ) {
		if ( true === $value ) {
			$value = 'true';
		}
		if ( false === $value ) {
			$value = 'false';
		}

		$out .= sprintf( ' %s="%s"', esc_html( $prefix . $key ), esc_attr( $value ) );
	}

	return $out;
}

/**
 * Remove 'bypostauthor' class from comment.
 *
 * @param  string $classes default classes.
 * @return string          updated classes.
 */
function books_filter_comment_class( $classes ) {

	if ( books_get_settings( 'author_comment' ) ) {
		return $classes;
	}

	if ( is_array( $classes ) ) {
		if ( count( $classes ) >= 1 ) {
			foreach ( $classes as $key => $class ) {
				if ( 'bypostauthor' === $class ) {
					unset( $classes[ $key ] );
				}
			}
		}
	}
	return $classes;
}
add_filter( 'comment_class', 'books_filter_comment_class' );

if ( ! function_exists( 'books_excluded_sidebars' ) ) {
	/**
	 * Get array of sidebar ids to exclude in options
	 *
	 * @return array
	 */
	function books_excluded_sidebars() {
		$ids = [
			'sidebar',
			'footer-top-1',
			'footer-top-2',
			'footer-top-3',
			'footer-top-4',
			'footer-top-5',
			'single-subscribe',
			'subscribe-section',
			'shop-sidebar',
			'product-sidebar',
		];

		return books_filter( 'exclude_sidebars', $ids );
	}
}

function validate_featured_categories( $field, $values ) {
	foreach ( $values as &$value ) {
		if ( ! isset( $value['unique_id'] ) || empty( $value['unique_id'] ) ) {
			$value['unique_id'] = uniqid();
		}
	}
	return [ 'value' => $values ];
}

/**
 * Handle AJAX search queries.
 */
function ajax_books_search() {
	$query        = $_REQUEST['q']; // It goes through esc_sql() in WP_Query.
	$search_query = new WP_Query( array( 's' => $query, 'posts_per_page' => 3, 'post_status' => 'publish' ) ); // @codingStandardsIgnoreLine
	$search_count = new WP_Query( array( 's' => $query, 'posts_per_page' => -1, 'post_status' => 'publish' ) ); // @codingStandardsIgnoreLine
	$search_count = $search_count->post_count;
	if ( ! empty( $query ) && $search_query->have_posts() ) :
		echo '<ul class="ajax-search-results">';
		while ( $search_query->have_posts() ) :
			$search_query->the_post();
		?>
			<li>
				<a href="<?php echo esc_url( get_the_permalink() ); ?>">
					<?php
					$attr = [
						'title'   => esc_attr( get_the_title() ),
						'alt'     => esc_attr( get_the_title() ),
						'loading' => 'lazy',
					];
					the_post_thumbnail( 'books-widgetthumb', $attr );
					?>
					<?php the_title(); ?>
				</a>
				<div class="meta">
					<span class="thetime"><?php the_time( 'F j, Y' ); ?></span>
				</div> <!-- / .meta -->
			</li>
			<?php
		endwhile;
		echo '</ul>';
		echo '<div class="ajax-search-meta"><span class="results-count">' . $search_count . ' ' . __( 'Results', 'books' ) . '</span><a href="' . esc_url( get_search_link( $query ) ) . '" class="results-link">' . __( 'Show all results.', 'books' ) . '</a></div>';
	else :
		echo '<div class="no-results">' . __( 'No results found.', 'books' ) . '</div>';
	endif;
	wp_reset_postdata();
	exit; // required for AJAX in WP.
}
add_action( 'wp_ajax_books_search', 'ajax_books_search' );
add_action( 'wp_ajax_nopriv_books_search', 'ajax_books_search' );

/**
 * Lightens/darkens a given colour (hex format), returning the altered colour in hex format.
 * @param str $hex Colour as hexadecimal (with or without hash);
 * @percent float $percent Decimal ( 0.2 = lighten by 20%(), -0.4 = darken by 40%() )
 * @return str Lightened/Darkend colour as hexadecimal (with hash);
 */
function color_luminance( $hex, $percent ) {

	// validate hex string

	$hex     = preg_replace( '/[^0-9a-f]/i', '', $hex );
	$new_hex = '#';

	if ( strlen( $hex ) < 6 ) {
		$hex = $hex[0] + $hex[0] + $hex[1] + $hex[1] + $hex[2] + $hex[2];
	}

	// convert to decimal and change luminosity
	for ( $i = 0; $i < 3; $i++ ) {
		$dec      = hexdec( substr( $hex, $i * 2, 2 ) );
		$dec      = min( max( 0, $dec + $dec * $percent ), 255 );
		$new_hex .= str_pad( dechex( $dec ), 2, 0, STR_PAD_LEFT );
	}

	return $new_hex;
}

/**
 * Convert hexdec color string to rgb(a) string
 *
 * @param str $color Colour as hexadecimal (with or without hash);
 * @param $opacity boolean
 *
 */
function hex2rgba( $color, $opacity = false ) {

	$default = 'rgb(0,0,0)';

	// Return default if no color provided.
	if ( empty( $color ) ) {
		return $default;
	}

	// Sanitize $color if "#" is provided.
	if ( '#' == $color[0] ) {
		$color = substr( $color, 1 );
	}

	// Check if color has 6 or 3 characters and get values.
	if ( 6 == strlen( $color ) ) {
		$hex = array( $color[0] . $color[1], $color[2] . $color[3], $color[4] . $color[5] );
	} elseif ( 3 == strlen( $color ) ) {
		$hex = array( $color[0] . $color[0], $color[1] . $color[1], $color[2] . $color[2] );
	} else {
		return $default;
	}

	// Convert hexadec to rgb.
	$rgb = array_map( 'hexdec', $hex );

	// Check if opacity is set(rgba or rgb).
	if ( $opacity ) {
		if ( abs( $opacity ) > 1 ) {
			$opacity = 1.0;
		}
		$output = 'rgba( ' . implode( ',', $rgb ) . ',' . $opacity . ' )';
	} else {
		$output = 'rgb( ' . implode( ',', $rgb ) . ' )';
	}

	// Return rgb(a) color string.
	return $output;
}

/**
 * Retrieve the sidebar layout
 *
 * @return string
 */
if ( ! function_exists( 'books_custom_sidebar_layout' ) ) {
	function books_custom_sidebar_layout() {
		//$full_sidebar = get_post_meta( get_the_ID(), '_mts_custom_sidebar', true );
		if ( is_singular() ) :
			$sidebar_layout = get_post_meta( get_the_ID(), '_mts_sidebar_location', true );
			switch ( $sidebar_layout ) {
				case 'left':
					$sidebar_layout = 'sclayout';
					break;

				case 'right':
					$sidebar_layout = 'cslayout';
					break;

				default:
					$sidebar_layout = books_get_settings( 'theme_layout' );
					break;
			}
			return $sidebar_layout;
		endif;
		//return $full_sidebar;
	}
}

if ( ! function_exists( 'books_register_elementor_locations' ) ) {
	/**
	 * Register Elementor Locations
	 */
	function books_register_elementor_locations( $elementor_theme_manager ) {
		if ( apply_filters( 'books_register_elementor_locations', true ) ) {
			$elementor_theme_manager->register_all_core_location();
			$elementor_theme_manager->register_location( 'archive' );
			$elementor_theme_manager->register_location(
				'main-sidebar',
				[
					'label'           => __( 'Main Sidebar', 'books' ),
					'multiple'        => true,
					'edit_in_content' => false,
				]
			);
		}
	}
}
add_action( 'elementor/theme/register_locations', 'books_register_elementor_locations' );

/**
 * Add span tag in category and archive count
 *
 * @param sting $widget_output input and output.
 */
function add_span_into_count( $widget_output ) {
	$widget_output = str_replace( '(', '<span>(&nbsp;', $widget_output );
	$widget_output = str_replace( ')', '&nbsp;)</span>', $widget_output );
	return $widget_output;
}
add_filter( 'wp_list_categories', 'add_span_into_count' );
add_filter( 'get_archives_link', 'add_span_into_count' );

/**
 * Create a dummy attribute.
 *
 * @since 2.3
 *
 * @param string        $raw_name Name of attribute to create.
 * @param array(string) $terms          Terms to create for the attribute.
 * @return array
 */
function create_attribute( $raw_name = 'Author Name', $terms = [] ) {
	if ( ! books_is_woocommerce_active() ) {
		return;
	}
	global $wpdb, $wc_product_attributes;

	// Make sure caches are clean.
	delete_transient( 'wc_attribute_taxonomies' );
	WC_Cache_Helper::invalidate_cache_group( 'woocommerce-attributes' );

	// These are exported as labels, so convert the label to a name if possible first.
	$attribute_labels = wp_list_pluck( wc_get_attribute_taxonomies(), 'attribute_label', 'attribute_name' );
	$attribute_name   = array_search( $raw_name, $attribute_labels, true );

	if ( ! $attribute_name ) {
		$attribute_name = wc_sanitize_taxonomy_name( $raw_name );
	}

	$attribute_id = wc_attribute_taxonomy_id_by_name( $attribute_name );

	if ( ! $attribute_id ) {
		$taxonomy_name = wc_attribute_taxonomy_name( $attribute_name );

		// De-register taxonomy which other tests may have created...
		unregister_taxonomy( $taxonomy_name );

		$attribute_id = wc_create_attribute(
			[
				'name'         => $raw_name,
				'slug'         => $attribute_name,
				'type'         => 'select',
				'order_by'     => 'menu_order',
				'has_archives' => 0,
			]
		);

		// Register as taxonomy.
		register_taxonomy(
			$taxonomy_name,
			apply_filters( 'woocommerce_taxonomy_objects_' . $taxonomy_name, [ 'product' ] ),
			apply_filters(
				'woocommerce_taxonomy_args_' . $taxonomy_name,
				[
					'labels'       => [
						'name' => $raw_name,
					],
					'hierarchical' => false,
					'show_ui'      => false,
					'query_var'    => true,
					'rewrite'      => false,
				]
			)
		);

		// Set product attributes global.
		$wc_product_attributes = [];

		foreach ( wc_get_attribute_taxonomies() as $taxonomy ) {
			$wc_product_attributes[ wc_attribute_taxonomy_name( $taxonomy->attribute_name ) ] = $taxonomy;
		}
	}

	$attribute = wc_get_attribute( $attribute_id );
	$return    = [
		'attribute_name'     => $attribute->name,
		'attribute_taxonomy' => $attribute->slug,
		'attribute_id'       => $attribute_id,
		'term_ids'           => [],
	];

	foreach ( $terms as $term ) {
		$result = term_exists( $term, $attribute->slug );

		if ( ! $result ) {
			$result               = wp_insert_term( $term, $attribute->slug );
			$return['term_ids'][] = $result['term_id'];
		} else {
			$return['term_ids'][] = $result['term_id'];
		}
	}

	return $return;
}

/**
 *  Array containing all Adsense Ad sizees [Used in Options Panel]
 *
 * @return array
 */
function ad_sizes() {
	$ad_sizes = array(
		'0'  => 'Auto',
		'1'  => '120 x 90',
		'2'  => '120 x 240',
		'3'  => '120 x 600',
		'4'  => '125 x 125',
		'5'  => '160 x 90',
		'6'  => '160 x 600',
		'7'  => '180 x 90',
		'8'  => '180 x 150',
		'9'  => '200 x 90',
		'10' => '200 x 200',
		'11' => '234 x 60',
		'12' => '250 x 250',
		'13' => '320 x 100',
		'14' => '300 x 250',
		'15' => '300 x 600',
		'16' => '300 x 1050',
		'17' => '320 x 50',
		'18' => '336 x 280',
		'19' => '360 x 300',
		'20' => '435 x 300',
		'21' => '468 x 15',
		'22' => '468 x 60',
		'23' => '640 x 165',
		'24' => '640 x 190',
		'25' => '640 x 300',
		'26' => '728 x 15',
		'27' => '728 x 90',
		'28' => '970 x 90',
		'29' => '970 x 250',
		'30' => '240 x 400',
		'31' => '250 x 360',
		'32' => '580 x 400',
		'33' => '750 x 100',
		'34' => '750 x 200',
		'35' => '750 x 300',
		'36' => '980 x 120',
		'37' => '930 x 180',
	);

	return $ad_sizes;
}

/**
 *  Returns Ad Width and Height
 *
 * @param $ad_size
 *
 * @return array
 */
if ( ! function_exists( 'ad_size_value' ) ) {
	function ad_size_value( $ad_size = '0' ) {
		$get_ad_size_array = ad_sizes();

		if ( 'Auto' != $get_ad_size_array[ $ad_size ] ) {
			$ad_size_parts = explode( ' x ', $get_ad_size_array[ $ad_size ] );
			$ad_size       = array(
				'ad_width'  => $ad_size_parts[0],
				'ad_height' => $ad_size_parts[1],
			);
			echo 'width:' . $ad_size['ad_width'] . 'px; height:' . $ad_size['ad_height'] . 'px;';
		}
	}
}
